<?php
/**
 * grid extension builder . use this class to create grid header and filter form
 * this class extends class grid_filter
 * @author Ihab A. Ali
 * @copyright copyright (c) copyright 2009 
 * @license GNU Public Licence (GPL)
 * to use the class :
 * @example $grid_extension = new grid_extension();
 * to initiate new filter and header :
 * @example $grid_extension->new_obj('menu_name');
 * to set parameters :
 * @example $grid_extension->set_title('$title');
 * @example $grid_extension->set_width(200);
 * to add the initiated header:
 * @example $grid_extension->add_obj();
 */
class grid_extension extends grid_filter {
    /**
     *
     * @var $show_in
     */
    private $show_in = 'both';
    /**
     * will be use to set the header
     * @access private
     * @var array $header
     */
    private $header     =   array();
    /**
     * will be use to set the filter
     * @access private
     * @var array $filter
     */
    private $filter     =   array();
    /**
     *
     * @param string $show_in
     */
    public function show_in($show_in = 'both') {
        $this->show_in = $show_in;
        return $this;
    }
    /**
     * set data base name
     * @access public
     * @param string $db_name
     * @param string $filter_alias
     */
    public function set_db_name($db_name,$filter_alias = null) {
        parent::set_db_name($db_name);
        if(!$filter_alias)$filter_alias = $db_name;
        parent::set_filter_alias($filter_alias);
        return $this;
    }
    /**
     * @access public
     * @param string $header_type
     * @param string $filter_type
     */
    public function set_type($header_type , $filter_type = null) {
        parent::set_header_type($header_type);
        if(!$filter_type)$filter_type = $header_type;
        parent::set_filter_type($filter_type);
        return $this;
    }
    /**
     *
     * @param string $header_alias
     * @param string $filter_alias
     */
    public function set_alias($header_alias , $filter_alias = null) {
        parent::set_header_alias($header_alias);
        if(!$filter_alias)$filter_alias = $header_alias;
        parent::set_filter_alias($filter_alias);
        return $this;
    }
    /**
     *
     * @param string $header_title
     * @param string $filter_title
     */
    public function set_title($header_title , $filter_title = null) {
        parent::set_header_title($header_title);
        if(!$filter_title)$filter_title = $header_title;
        parent::set_filter_title($filter_title);
        return $this;
    }
    /**
     *
     * @param int $header_width
     * @param int $filter_width
     */
    public function set_width($header_width , $filter_width = 180) {
        parent::set_header_width($header_width);
        parent::set_filter_width($filter_width);
        return $this;

    }
    /**
     * unset all variables
     */
    private function unset_all() {
        unset($this->filter_alias);
        unset($this->filter_title);
        unset($this->filter_type);
        unset($this->object);

        unset($this->header_alias);
        unset($this->db_name);
        unset($this->header_title);
        unset($this->header_type);
        parent::set_header_align()
                ->set_column_align()
                ->set_min_width()
                ->set_is_resizable()
                ->set_header_width();


        $this->show_in();

    }
    /**
     * @access public
     * @param string $db_name
     */
    public function new_obj($db_name=null) {
        $this->unset_all();
        $this->set_db_name($db_name);
        return $this;
    }
    /**
     *
     */
    public function show_numbering() {
        $this->new_obj("numbering")
                ->set_title("#")
                ->set_width(40)
                ->set_min_width(40)
                ->set_is_resizable(false)
                ->set_type("numbering")
                ->show_in('header')
                ->set_header_align("center")
                ->set_column_align("center")
                ->add_obj();
        return $this;
    }
    /**
     *
     * @param string $id
     */
    public function set_modify_id($id=array()) {
        $this->new_obj("modify_id");
        $this->set_title(' ');
        $this->set_filter_type("extend_link");
        if(is_array($id))
            $this->set_filter_alias(implode('&',preg_replace('/"/', "/'/", $id)));
        else
            $this->set_filter_alias($id);

        $this->show_in('filter');
        $this->add_obj();

        return $this;
    }
    /**
     * add object in arrays
     * @access public
     * @param bool $header
     * @param bool $filter
     */
    public function add_obj() {
        if($this->show_in == 'header' OR $this->show_in == 'both') {
            $var_header = array('db_name'=>parent::get_db_name(),
                    'title'=>parent::get_header_title(),
                    'width'=>parent::get_header_width(),
                    'min_width'=>parent::get_min_width(),
                    'is_resizable'=>parent::get_is_resizable(),
                    'type'=>parent::get_header_type(),
                    'alias'=>parent::get_header_alias(),
                    'header_align'=>parent::get_header_align(),
                    'column_align'=>parent::get_column_align()
            );
            array_push($this->header, $var_header);
        }
        if($this->show_in == 'filter' OR $this->show_in == 'both') {
            if(!parent::get_filter_alias())
                parent::set_filter_alias(parent::get_db_name());

            $var_filter = array('db_name'=>parent::get_db_name(),
                    'title'=>parent::get_filter_title(),
                    'type'=>parent::get_filter_type(),
                    'width'=>parent::get_filter_width(),
                    'object'=>parent::get_object(),
                    'alias'=>parent::get_filter_alias()
            );
            array_push($this->filter, $var_filter);
        }
        $this->unset_all();
        return $this;
    }
    /**
     *
     * @return $header
     */
    public function get_header() {
        return $this->header;
    }
    public function get_filter() {
        $arr_string = array(
                "LIKE"=>SELECT_LIKE,
                "NOT LIKE"=>SELECT_NOT_LIKE,
                "="=>"=",
                "!="=>"!=",
                "IS NULL"=>SELECT_NULL,
                "IS NOT NULL"=>SELECT_NOT_NULL
        );
        $arr_int = array(
                "="=>"=",
                ">"=>">",
                ">="=>">=",
                "<"=>"<",
                "<="=>"<=",
                "!="=>"!="
        );

        $arr_bool     =   array(0=>SELECT_NO,1=>SELECT_YES);

        $arr_gender   =   array('M'=>SELECT_MALE,'F'=>SELECT_FEMALE);

        $arr_mutli  =   array(
                "IN"=>SELECT_IN,
                "NOT IN"=>SELECT_NOT_IN
        );
        $table = new table();
        $table
                ->set_cell_spacing(6)
                ->set_elements_per_row(2);
        $params = array();
        $arr_fields = array();
        foreach(@$this->filter as $filter) {
            array_push($arr_fields, $filter['db_name']);

            switch($filter['type']) {
//                case 'attachment':
//                    $table->span(3);
//                    
//                    $elements  .= '<td><div  id="'.$filter['db_name'].'__div">&nbsp;'.$filter['alias'].'</div></td>';
//                    
//                    $table->new_element($filter['title'],"<table ><tr valign='top'><a href='#'>'{$elements}'<a></tr></table>");
//                    
//                    array_push($params,$filter['db_name']."__type:'string'");
//                    array_push($params,$filter['db_name']."__like:{$filter['db_name']}__like.getActualValue()");
//                    array_push($params,$filter['db_name']."__value:$('#".$filter['db_name']."').val()");
//                    array_push($params,$filter['db_name']."__alias:'".$filter['alias']."'");
//                    break;
                case 'extend_link':
                    array_push($params,"extend_link:'{$filter['alias']}'");
                    break;
                case 'string':
                    $string_select = new combo($filter['db_name'].'__like',"0","1");
                    $string_select->set_data_source($arr_string, "array");
                    $string_select->set_width('80');
                    $string_select->set_readonly();
                    $elements   = '<td>'.$string_select->load().'</td>';
                    $input_text = new input_text($filter['db_name']);
                    $input_text->set_width($filter['width'].'px');
                    $elements  .= '<td><div  id="'.$filter['db_name'].'__div">&nbsp;'.$input_text->load().'</div></td>';
                    $table->span(3);
                    $table->new_element($filter['title'],"<table ><tr valign='top'>".$elements."</tr></table>");

                    array_push($params,$filter['db_name']."__type:'string'");
                    array_push($params,$filter['db_name']."__like:{$filter['db_name']}__like.getActualValue()");
                    array_push($params,$filter['db_name']."__value:$('#".$filter['db_name']."').val()");
                    array_push($params,$filter['db_name']."__alias:'".$filter['alias']."'");
                    break;
                case 'text':
                    $string_select = new combo($filter['db_name'].'__like');
                    $string_select->set_data_source($arr_string, "array");
                    $string_select->set_width('80');
                    $string_select->set_readonly();
                    $elements  = '<td>'.$string_select->load().'</td>';
                    $textarea = new textarea($filter['db_name']);
                    $textarea->set_width($filter['width']."px");
                    $elements  .='<td><div id="'.$filter['db_name'].'__div">&nbsp;'.$textarea->load().'</div></td>';
                    $table->span(3);
                    $table->new_element($filter['title'], '<table ><tr valign="top">'.$elements.'</tr></table>');

                    array_push($params,$filter['db_name']."__type:'text'");
                    array_push($params,$filter['db_name']."__like:{$filter['db_name']}__like.getActualValue()");
                    array_push($params,$filter['db_name']."__value:$('#".$filter['db_name']."').val()");
                    array_push($params,$filter['db_name']."__alias:'".$filter['alias']."'");

                    break;

                case 'int':
                    $int_select_1 = new combo($filter['db_name'].'__like__1');
                    $int_select_1->set_data_source($arr_int, "array");
                    $int_select_1->set_readonly();
                    $int_select_1->set_width('80');
                    $input_mask_1 = new input_mask($filter['db_name'].'__1','int');
                    $input_mask_1->set_width($filter['width'].'px');
                    $element_1 = '<td>'.$int_select_1->load().'</td>';
                    $element_1 .='<td><div id="'.$filter['db_name'].'__div__1">&nbsp;'.$input_mask_1->load().'</div></td>';
                    $table->new_element($filter['title'], '<table ><tr valign="top">'.$element_1.'</tr></table>');
                    $int_select_2 = new combo($filter['db_name'].'__like__2');
                    $int_select_2->set_data_source($arr_int, "array");
                    $int_select_2->set_readonly();
                    $int_select_2->set_width(80);
                    $input_mask_2 = new input_mask($filter['db_name'].'__2','int');
                    $input_mask_2->set_width( $filter['width'].'px');
                    $element_2  = '<td>'.$int_select_2->load().'</td>';
                    $element_2 .= '<td><div style="float:'.DIR_ALIGN.';" id="'.$filter['db_name'].'__div__2">&nbsp;'.$input_mask_2->load().'</div></td>';
                    $table->new_element(FILTER_AND, '<table ><tr valign="top">'.$element_2.'</tr></table>');

                    array_push($params,$filter['db_name']."__type:'int'");
                    array_push($params,$filter['db_name']."__like__1:{$filter['db_name']}__like__1.getActualValue()");
                    array_push($params,$filter['db_name']."__value__1:$('#".$filter['db_name']."__1').val()");
                    array_push($params,$filter['db_name']."__like__2:{$filter['db_name']}__like__2.getActualValue()");
                    array_push($params,$filter['db_name']."__value__2:$('#".$filter['db_name']."__2').val()");
                    array_push($params,$filter['db_name']."__alias:'".$filter['alias']."'");

                    break;
                case 'date':
                    $int_select_1 = new combo($filter['db_name'].'__like__1');
                    $int_select_1->set_data_source($arr_int, "array");
                    $int_select_1->set_readonly();
                    $int_select_1->set_width(80);
                    $calendat_1 = new calendar($filter['db_name'].'__1');
                    $calendat_1->set_width($filter['width'].'px');
                    $element_1 = '<td>'.$int_select_1->load().'</td>';
                    $element_1 .= '<td>'.$calendat_1->load().'</td>';
                    $table->new_element($filter['title'], '<table ><tr valign="top">'.$element_1.'</tr></table>');

                    $int_select_2 = new combo($filter['db_name'].'__like__2');
                    $int_select_2->set_data_source($arr_int, "array");
                    $int_select_2->set_readonly();
                    $int_select_2->set_width(80);
                    $calendat_2 = new calendar($filter['db_name'].'__2');
                    $calendat_2->set_width($filter['width'].'px');
                    $element_2 = '<td>'.$int_select_2->load().'</td>';
                    $element_2 .= '<td>'.$calendat_2->load().'</td>';
                    $table->new_element(FILTER_AND, '<table ><tr valign="top">'.$element_2.'</tr></table>');

                    array_push($params,$filter['db_name']."__type:'date'");
                    array_push($params,$filter['db_name']."__like__1:{$filter['db_name']}__like__1.getActualValue()");
                    array_push($params,$filter['db_name']."__value__1:$('#".$filter['db_name']."__1').val()");
                    array_push($params,$filter['db_name']."__like__2:{$filter['db_name']}__like__2.getActualValue()");
                    array_push($params,$filter['db_name']."__value__2:$('#".$filter['db_name']."__2').val()");
                    array_push($params,$filter['db_name']."__alias:'".$filter['alias']."'");

                    break;
                case 'multi':
                    $multi_select = new combo($filter['db_name'].'__like',"0","1");
                    $multi_select->set_data_source($arr_mutli, "array");
                    $multi_select->set_readonly();
                    $multi_select->set_width(80);
                    $filter['object']->set_width($filter['width']);
                    $elements  = $multi_select->load();
                    $elements .='<div id="'.$filter['db_name'].'__div">'.$filter['object']->load().'</div>';
                    $table->span(3);
                    $table->new_element($filter['title'], $elements);

                    array_push($params,$filter['db_name']."__type:'multi'");
                    array_push($params,$filter['db_name']."__like:{$filter['db_name']}__like.getActualValue()");
                    array_push($params,$filter['db_name']."__value:$('#".$filter['object']->get_name()."').val()");
                    array_push($params,$filter['db_name']."__alias:'".$filter['alias']."'");
                    break;
                case 'bool':
                    $bool = new combo($filter['db_name']);
                    $bool->set_data_source($arr_bool, "array");
                    $bool->set_width($filter['width']);
                    $table->span(3);
                    $table->new_element($filter['title'], $bool->load());

                    array_push($params,$filter['db_name']."__type:'bool'");
                    array_push($params,$filter['db_name']."__value:".$bool->get_name().".getActualValue()");
                    array_push($params,$filter['db_name']."__alias:'".$filter['alias']."'");

                    break;
                case 'gender':
                    $gender = new combo($filter['db_name'],0,1);
                    $gender->set_data_source($arr_gender, "array");
                    $gender->set_width($filter['width']);
                    $table->span(3);
                    $table->new_element($filter['title'], $gender->load());

                    array_push($params,$filter['db_name']."__type :'gender'");
                    array_push($params,$filter['db_name']."__value:".$gender->get_name().".getActualValue()");
                    array_push($params,$filter['db_name']."__alias:'".$filter['alias']."'");

                    break;
                case 'combo':
                    $table->span(3);
                    $filter['object']->set_width($filter['width']);
                    $table->new_element($filter['title'], $filter['object']->load());
                    array_push($params,$filter['db_name']."__type :'list'");
                    array_push($params,$filter['db_name']."__value:".$filter['object']->get_name().".getActualValue()");
                    array_push($params,$filter['db_name']."__alias:'".$filter['alias']."'");
                    break;
                case 'select':
                    $table->span(3);
                    $filter['object']->set_width($filter['width']);
                    $table->new_element($filter['title'], '<div id="'.$filter['db_name'].'__div">'.$filter['object']->load().'</div>');

                    array_push($params,$filter['db_name']."__type:'list'");
                    array_push($params,$filter['db_name']."__value:$('#".$filter['object']->get_name()."').val()");
                    array_push($params,$filter['db_name']."__alias:'".$filter['alias']."'");
                    break;
                case 'radio':
                    $table->span(3);
                    if(is_array($filter['object'])) {
                        $object = new combo($filter['db_name']);
                        $object->set_data_source($filter['object'],'array');
                        $object->set_width($filter['width']);
                        $table->new_element($filter['title'], $object->load());
                        array_push($params,$filter['db_name']."__type :'list'");
                        array_push($params,$filter['db_name']."__value:".$object->get_name().".getActualValue()");
                        array_push($params,$filter['db_name']."__replace_value:'".json_encode($filter['object'])."'");
                        array_push($params,$filter['db_name']."__alias:'".$filter['alias']."'");
                    }
                    break;
            }

        }

        array_push($params,"filter_elements:".json_encode($arr_fields));
        $table->set_title(LANG_FILTER_FORM);
        $array = array(

                'form'=>$table->load(LANG_FILTER_FORM),
                'params'=>$params
        );


        return $array;
    }

}
?>