<?php

/**
 * select data_source . use this class to call for the select or combo source options
 * inherited from sql_class
 * @example  $html = new data_source($name,$option_view=null,$option_value=null);
 */
class data_source extends builder_elements {

    /**
     *
     * @var string  $db_table_name
     */
    private $debug = false;

    /**
     *
     * @var object sql
     */
    private $sql = null;

    /**
     *
     * @var string option_value
     */
    private $option_value = null;

    /**
     *
     * @var option_view
     */
    private $option_view = null;

    /**
     * enum of type query , results array or text file location
     * @var string type
     */
    private $source_type = 'query';

    /**
     * it could be a sql query , a results array ,or text file location
     * @var source
     */
    private $source_var = null;

    /**
     *
     * @var array data set
     */
    public $is_data_field = false;

    /**
     *
     * @var array data set
     */
    private $data_set = array();

    /**
     * the class constructor
     * @access public
     *
     */
    public function __construct($option_view, $option_value, $sql_obj = null) {
        $this->set_option_view($option_view);
        $this->set_option_value($option_value);
        $this->set_sql($sql_obj);
    }

    /**
     * here if we want from the data source to use not new sql object but exist object
     * or object with connect to other database or different user/pass ..
     * 
     * @param type $sql_obj
     * @return \data_source
     */
    public function set_sql($sql_obj = null) {
        $this->sql = $sql_obj ? $sql_obj : new sql();
        return $this;
    }
    public function get_sql(){
        return $this->sql;
    }

    /**
     * @access public
     * @param string $debug
     * @return none
     */
    public function debug($debug = true) {
        $this->debug = $debug;
        return $this;
    }

    /**
     * @access public
     * @return get_debug
     */
    public function get_debug() {
        return $this->debug;
    }

    /**
     * @access public
     * @param <type> $option_value
     */
    public function set_option_value($option_value) {
        $this->option_value = $option_value;
        return $this;
    }

    /**
     * @access public
     * @return string $option_value
     */
    public function get_option_value() {
        return $this->option_value;
    }

    /**
     * @access public
     * @param string $option_view
     */
    public function set_option_view($option_view) {
        $this->option_view = $option_view;
        return $this;
    }

    /**
     * @access public
     * @return string $option_view
     */
    public function get_option_view() {
        return $this->option_view;
    }

    /**
     *
     * @param type $option_value
     * @param type $option_view
     * @return \data_source 
     * this must be done after set_data_source
     */
    public function add_option($option_value, $option_view) {
        $this->data_set[$option_value] = $option_view;
        return $this;
    }

    /**
     * @access public
     * @param enum $source_type 'query','array()','text'
     */
    private function set_source_type($type = 'query') {
        $this->source_type = $type;
        return $this;
    }

    /**
     * @access public
     * @return string $source_type
     */
    public function get_source_type() {
        return $this->source_type;
    }

    /**
     * @access public
     * @param string $source_var
     */
    private function set_source_var($source_var = null) {
        $this->source_var = $source_var;
        return $this;
    }

    /**
     * @access public
     * @return string $source_var
     */
    public function get_source_var() {
        return $this->source_var;
    }

    /**
     *
     * @param string $source_var
     * @param string $type
     */
    public function set_data_source($source_var, $type = "query") {
        $this->set_source_type($type);
        $this->set_source_var($source_var);
        switch ($this->get_source_type()) {
            case 'query': $this->load_query_data();
                break;
            case 'array': $this->set_data_set($this->get_source_var());
                break;
            case 'file' : $this->load_file_data();
                break;
        }
        return $this;
    }

    /**
     *
     * @param array $data_set
     */
    private function set_data_set($data_set = null) {
        $this->data_set = $data_set;
        return $this;
    }

    /**
     *
     * @return array $data_set
     */
    public function get_data_set($key = null) {
        if (is_null($key))
            return $this->data_set;
        return $this->data_set[$key];
    }

    //this return must used in the data source as concat of option view value & the return string here
    public static function build_sql_combo_fld($key, $db_fld) {
        //it will be something like concat(record_name,$this_ft_return ..[,$optnal_another_key_this_ft_rtrn])
        return "'<span rf={$key} val=\"',replace(replace({$db_fld}, \"'\", '&#039;'), '\"', '&quot;'),'\"></span>'";
    }

    //
    public function get_data_combo_fld($db_val, $fld_key) {
        $data_val = $this->get_data_set($db_val);
        $search_key = "rf={$fld_key}";
        $start_pos = strpos($data_val, $search_key) + strlen($search_key);
        $fld_val = trim(substr($data_val, $start_pos)); //here since maybe he write rf=xxx   val=.. here maybe before val he make many spaces..
        $matches = array();
        if (preg_match("/val=\"(.*?)\"/", $fld_val, $matches))
            return str_replace(array("&quot;", "&#039;"), array('"', "'"), $matches[1]); //since in above ft. "build_sql_combo_fld" we replace it
        return "";
    }

    /**
     *
     */
    private function load_file_data() {
        $data = $this->data_from_file($this->get_source_var());
        $data = split("\n", $data);
        $data_source = array();
        foreach ($data as $value) {
            $value = explode("\t", $value);
            $data_source[$value[$this->get_option_value()]] = $value[$this->get_option_view()];
        }
        $this->set_data_set($data_source);
    }

    /**
     *
     * @return <type>
     */
    private function load_query_data() {
        $options = $this->sql->get_result($this->get_source_var(), $this->get_debug());
        $data_source = array();
        while ($row = $this->sql->next($options)) {
            if ($this->is_data_field)
                $data_source[$row[$this->get_option_value()]] = $row;
            else
                $data_source[$row[$this->get_option_value()]] = $row[$this->get_option_view()];
        }
        $this->set_data_set($data_source);
    }

    /**
     * @access private
     * @param string $src
     * @return string $filedata
     */
    private function data_from_file($src) {
        $buffer = fopen("$src", "rw") or die("can't open file");
        $content = fread($buffer, filesize("$src"));
        fclose($buffer);
        return $content;
    }

}

?>