<?php

/**
 * @example  $calendar = new calendar("calendarname");
 *           $calendar->setformat("Y-m-d");
 *           $calendar->set_onchange("function()");set js function on change
 *           $calendar->setwidth("180");
 *           $calendar->setname("calendarname"); used in construct;
 */
class calendar extends builder_elements {

    const strtime = "His";
    const strdate = "YMmd";
    const strday = "Ymd";
    const strdatetime = "YMmdHis";
    const strslct_time = "MHis";
    const default_date = "Y-m-d";

    private $direction = 'ltr';
    private $onchange = null;
    private $time = array();
    private $time_newline = false;
    private $time_isdefault = true;
    private $hide_time = false;

    /**
     * construct function
     * @method public
     * @param string $name to set the name
     */
    public function __construct($name) {
        /* set the name of calendar */
        parent::set_name($name);
//        parent::set_valid_msg(LANG_VALID);

        /* to enable use it outside framework since authenticate class might not exist */
        if (class_exists("authentication"))/* check that class exist */
            $local_format = authentication::get_setting_calendar_format();
        if (!$local_format) /* if class exist not filled fill it with default */
            $local_format = self::default_date;
        $this->set_direction();
        $this->set_format($local_format);
    }

    /**
     * like when used for front-end
     */
    public function set_direction($dir = dir) {
        if ($dir == 'rtl' || $dir == 'ltr')
            $this->direction = $dir;
        return $this;
    }

    public function get_direction() {
        return $this->direction;
    }

    /**
     * if this is set & time exist then the select boxes of time will be exist but div is hidden
     */
    public function hide_time($hide_time = true) {
        $this->hide_time = $hide_time;
        return $this;
    }

    /**
     *
     * @return type 
     */
    private function get_hide_time() {
        return $this->hide_time;
    }

    /**
     *
     * @param type $new_line
     * @return \calendar 
     */
    public function set_time_newline($newline = true) {
        $this->time_newline = $newline;
        return $this;
    }

    /**
     * 
     */
    private function get_time_newline() {
        return $this->time_newline;
    }

    /**
     *
     * @param type $time_isdefault 
     */
    public function set_time_isdefault($time_isdefault = true) {
        $this->time_isdefault = $time_isdefault;
    }

    /**
     * 
     */
    private function get_time_isdefault() {
        return $this->time_isdefault;
    }

    /**
     * 
     */
    private static function is_ie() {
        return preg_match("/msie/", strtolower($_SERVER['HTTP_USER_AGENT']));
    }

    /**
     * 
     */
    private static function check_ptr($ptrn, $subj, $is_all = false) {
        if ($is_all) {
            if (preg_match_all("/[{$ptrn}]/", $subj, $return))
                return $return[0];
            return 0;
        } else
            return preg_match("/[{$ptrn}]/", $subj);
    }

    /**
     *
     * @param string $format
     * set the format of the calendar to Y-m-d |Y-d-m | d-m-Y |m-d-Y
     */
    public function set_format($format = 'Y-m-d') {
        /* reset the time pattern */
        $this->time = array();
        $this->time["fr_date"] = $format;
        /* check if match date */
        $is_date = self::check_ptr(self::strdate, $format, true);
        if ($is_date) {
            $this->time["is_date"] = true;
            $this->time["no_date"] = count($is_date);
        }
        /* check if match date */
        $is_time = self::check_ptr(self::strtime, $format, true);
        if ($is_time) {
            $this->time["is_time"] = true;
            $this->time["no_time"] = count($is_time);
        }
        /* check if match date time */
        $is_all = self::check_ptr(self::strdatetime, $format, true);
        if ($is_all)
            $this->time["fr_mask"] = $is_all;

        return $this;
    }

    /**
     *
     * @return type 
     */
    private function is_date() {
        return $this->time["is_date"];
    }

    /**
     * 
     */
    private function get_no_date() {
        return $this->time["no_date"];
    }

    /**
     *
     * @return type 
     */
    private function is_time() {
        return $this->time["is_time"];
    }

    /**
     * 
     */
    private function get_no_time() {
        return $this->time["no_time"];
    }

    /**
     *
     * @return string $format
     */
    public function get_date_format() {
        return $this->time["fr_date"];
    }

    /**
     *
     * @return string $format
     */
    private function get_mask_format() {
        return $this->time["fr_mask"];
    }

    /**
     *
     * @param type $mask
     * @return type 
     */
    private function get_src($mask, $label) {
        if (self::check_ptr(self::strslct_time, $mask)) {
            $start = 0;
            $total = 60;
            if ($mask == "H")/* if time is hour */
                $total = 24;
            else if ($mask == "M") {/* if month is Letters */
                $start = 1;
                $total = 13;
            }
            $option = "";
            /* handle arrays for time manage */
            if ($this->get_time_isdefault() && $mask != "M")
                $option .= "<option value=''>{$label}</option>";
            for ($i = $start; $i < $total; $i++) {
                $val = sprintf("%02s", $i);
                $view = $val;
                if ($mask == "M")
                    $view = get_define("df_mnth{$i}");
                $option .= "<option value='{$val}'>{$view}</option>";
            }
            return $option;
        }
        return 0;
    }

    /**
     * 
     */
    private function html_element($mask, $maxlength) {
        /* label to be used to set value */

        $label = get_define($mask);
        /* source related to check if pattern related to select box or input */
        $source = $this->get_src($mask, $label);
        /* if big M means select box but we need to se it to small m as an id reference */
        if ($mask == "M")
            $mask = "m";
        /* get id */
        $id = parent::get_id() . "_{$mask}";
        /* common style */
        /* common style */
        $attr = " maxlength='{$maxlength}' id='{$id}' style='{$style}' label='{$label}' value='{$label}' placeholder='{$label}' title='' class='format-int'";
        /* retrun the html element */
        return $html = ($source) ? "<select {$attr}>{$source}</select>" : "<input  type='text' autocomplete='off' {$attr} />";
    }

    /**
     *
     * set the elements according to format set
     */
    private function set_mask() {
        /* initialize html */
        $html = array();
        if ($this->get_mask_format()) { /* check if mask exist */
            /* loop on mask filter */
            $html["date"] = array();
            $html["time"] = array();
            foreach ($this->get_mask_format() as $key => $mask) {
                $maxlength = 2;
                $cor_by = self::check_ptr(self::strday, $mask) ? (self::is_ie() ? 6 : 4) : 2;
                $sub_by = 1;
                /* if mask is year assign as if 2 flds */
                if ($mask == 'Y') {
                    $sub_by = 2;
                    $maxlength = 4;
                }

                /* assign html element */
                array_push($html[(self::check_ptr(self::strdate, $mask) ? "date" : "time")], $this->html_element($mask, $maxlength));
            }
        }
        $html["date"] = implode("<span class='_date_time_sep'>/</span>", $html["date"]);
        $html["time"] = implode("<span class='_date_time_sep'>:</span>", $html["time"]);
        return $html;
    }

    /**
     * set a function script on calendar change
     */
    public function set_onchange($onchange = null) {
        $this->onchange .= $onchange;
        return $this;
    }

    private function get_onchange() {
        return $this->onchange;
    }

    /**
     *
     * @return the calendar as string in order to echo it
     */
    public function load() {
        if (parent::get_disable()) {
            anti_inspection::push_disabled_elments(parent::get_name(), parent::get_value(),"calendar");
        }
        $span = $this->get_required_span();

        $html = $this->set_mask(); /* html after building date and time formatting */
        if ($this->is_date()) {/* place calendar btn if is date */
            //float:" . ($this->get_direction() == 'rtl' ? 'right' : 'left') . "
            $btn = "<a tabindex='-1' href='javascript:;' id='{$this->get_id()}_button' class='mdi mdi-calendar-month calendar'></a>";
            /* html */
            $html_date = "<div id='{$this->get_id()}_mask_date_container'>{$html["date"]}{$btn}</div>";
        }
        if ($this->is_time()) {/* place is time of exist */
            /* style */
            $style_time = "display:inline-flex";
            if ($this->is_date()) {
                if ($this->get_hide_time())
                    $style_time = "display:none;";
                if ($this->get_time_newline())
                    $style_time = "display:flex;margin-top:5px;";
            }
            /* html */
            $html_time = ""
                    . "<div style='width:4px;display:inline-flex;'></div>"
                    . "<div id='{$this->get_id()}_mask_time_container' style='{$style_time}'>{$html["time"]}"
                    . "<span class='mdi mdi-clock-outline'></span>"
                    . "</div>";
        }
        /* html parse */
        $sub_div_display = $this->get_time_newline() ? 'block' : 'inline-flex';
        $html = "<div id='{$this->get_id()}_container' class='date_calendar' title='{$this->get_title()}'>
                    {$html_date}<input subtype='calendar' type='hidden' name='{$this->get_name()}' id='{$this->get_id()}' class='date'/>
                    {$html_time}
                    {$span}
                 </div>
                 <script type='text/javascript'>
                            /*function to be used in the init of the class*/
                            function {$this->get_id()}custom(){ 
                                {$this->get_onchange()} 
                                return false;
                            }
                            /*build in the calendar*/
                            $(document).ready(function(){
                                $('#{$this->get_id()}').date_int('{$this->get_id()}custom()'," . (($this->get_direction() == 'rtl') ? 'true' : 'false') . ")
                                    .date_val('{$this->get_value()}')
                                    .date_disable(" . ($this->get_disable() ? 'true' : 'false') . ")
                                    .valid(" . ($this->get_valid() ? 'true' : 'false') . ");
                            });
                 </script>";
        return $html;
    }

}

?>