<?php

/**
 * Description of builder_elements
 *
 * @author ihab
 */
class parser_dom extends table {

    protected $name = null;
    protected $element = array();
    protected $label_element = array();
    protected $insert_field = array();
    protected $button = array();
    protected $header = null;
    protected $alias_element = null;
    protected $db_table_name = null;
    protected $join = array();
    protected $order = null;
    protected $select_field = null;
    protected $parent_id = null;
    protected $sql = null;
    protected $allow_add = true;
    protected $allow_modify = true;
    protected $allow_new = true;
    protected $debug = false;
    protected $options = array();
    protected $onchange = null;
    protected $onadd = null;
    protected $onmodify = null;
    protected $onnew_record = null;
    protected $onload = null;
    protected $script = array();
    protected $before_delete = array();
    protected $ondelete = null;
    protected $after_delete = array();
    protected $after_add = array();
    protected $before_copy = array();
    protected $oncopy = null;
    protected $after_copy = array();
    protected $before_show_modify_form = array();
    protected $onshow_modify_form = null;
    protected $after_show_modify_form = array();
    protected $auto_hide_form = false;
    protected $form_display = "none";
    protected $hidden_inputs = array();

    /**
     * set the dom
     * @param string $name
     * create an sql instance to allow database set
     */
    public function __construct($name) {

        parent::__construct();

        $this->set_min_col_width(60)
                ->set_copy_title()
                ->set_modify_title()
                ->set_delete_title()
                ->set_delete_confirmation_msg()
                ->set_dir()
                ->set_name($name)
                ->set_width(500)
                ->override_onshow_modify_form(" dom.setParentData(el); ")
                ->override_oncopy(" dom.copy(el); ")
                ->override_before_delete("dom.setSelectedRow(el)")
                ->override_ondelete("dom.deleteAction(el)");
        parent::add_attr('id', $this->get_name() . '_display'); //give it an id;
        parent::set_elements_per_row(1);
        parent::set_cell_spacing(2);
        parent::set_spacer_width(120);

        $this->sql = new sql(); //create an instance of sql to deal with db;
    }

    /**
     * debug
     */
    public function debug($debug = true) {
        $this->debug = $debug;
        return $this;
    }

    public function get_debug() {
        return $this->debug;
    }

    /**
     * @param string $name
     * set dom name
     */
    public function add_button($button) {
        @array_push($this->button, $button);
        return $this;
    }

    /**
     * @method public
     * @return string $name of dom object
     */
    public function get_button() {
        return $this->button;
    }

    /**
     * @param string $name
     * set dom name
     */
    public function set_name($name) {
        $this->name = $name;
        return $this;
    }

    /**
     * @method public
     * @return string $name of dom object
     */
    public function get_name() {
        return $this->name;
    }

    public function disable_new($allow_new = false) {
        $this->allow_new = $allow_new;
        return $this;
    }

    /**
     * @method public
     * @return string $name of dom object
     */
    protected function get_allow_new() {
        return $this->allow_new;
    }

    public function enable_auto_hide_form($auto_hide_form = true) {
        $this->auto_hide_form = $auto_hide_form;
        return $this;
    }

    /**
     * @method public
     * @return string $name of dom object
     */
    protected function get_auto_hide_form() {
        return $this->auto_hide_form;
    }

    public function disable_add($allow_add = false) {
        $this->allow_add = $allow_add;
        $this->options['disableCopy'] = $allow_add;
        return $this;
    }

    /**
     * @method public
     * @return string $name of dom object
     */
    protected function get_allow_add() {
        return $this->allow_add;
    }

    /**
     * @method public
     * @return string $name of dom object
     */
    public function disable_modify($allow_modify = false) {
        $this->allow_modify = $allow_modify;
        $this->options['disableModify'] = $allow_modify;

        return $this;
    }

    /**
     * @method public
     * @return string $name of dom object
     */
    protected function get_allow_modify() {
        return $this->allow_modify;
    }

    public function disable_delete($allow_delete = false) {
        $this->options['disableDelete'] = $allow_delete;
        return $this;
    }

    /**
     *
     * @param string  $element
     * @return string
     * get the element when loop the array of the dom elements in html drawing
     */
    protected function get_element($element = null) {
        if ($element)
            return $this->element[$element];
        return $this->element;
    }

    /**
     *
     * @param string $key
     * @return string
     * get the alias element for labels in case of modify form
     */
    protected function get_alias_element($key = null) {
        if ($key)
            return $this->alias_element[$key];
        return $this->alias_element;
    }

    /**
     *
     * @return the select fields of  statment for the query on modify form
     */
    protected function get_select_field() {
        return substr($this->select_field, 0, -1);
    }

    /**
     *
     * @return string
     * set the insert statment for each array set in post action;
     */
    protected function get_insert_field() {
        return $this->insert_field;
    }

    /**
     *
     * @param int $id
     * set the id from parent table to child table
     */
    public function set_parent_id($id) {
        $this->parent_id = $id;

        return $this;
    }

    /**
     *
     * @return string
     * get the field key name of the parent table
     */
    public function get_parent_id() {
        return $this->parent_id;
    }

    /**
     *
     * @param string $table
     * set the table of the dom object
     */
    public function set_db_table_name($table) {
        $this->db_table_name = $table;
        return $this;
    }

    /**
     *
     * @return string
     * get the table name of the dom object
     */
    public function get_db_table_name() {
        return $this->db_table_name;
    }

    /**
     *
     * @param string $statment
     * set the from in select statment to enable the select of alias fields in modify form
     */
    public function add_join($statment) {
        array_push($this->join, $statment);

        return $this;
    }

    /**
     *
     * @return string
     * get the from statment to build the query
     */
    protected function get_join() {
        return implode(" ", $this->join);
    }

    public function set_order($order) {
        $this->order .= $order;

        return $this;
    }

    /**
     *
     * @return string
     * get the from statment to build the query
     */
    protected function get_order() {
        return $this->order;
    }

    /**
     *
     * @param string $script
     * set the js on case of add and modify and place condition on add click and modify
     */
    public function add_script($script) {
        $this->script[] = $script;
        return $this;
    }

    /**
     *
     * @return the script event set
     */
    protected function get_script() {
        return implode("; ", $this->script);
    }

    /**
     *
     * @param string $onchange
     * set the js on case of add and modify and place condition on add click and modify
     */
    public function set_onchange($onchange) {
        $this->onchange = $onchange;
        return $this;
    }

    /**
     *
     * @return the onchange event set
     */
    public function get_onchange() {
        return $this->onchange;
    }

    /**
     *
     * @param string $onload
     * set the js on case of add and modify and place condition on add click and modify
     */
    public function set_onload($onload) {
        $this->onload = $onload;
        return $this;
    }

    /**
     *
     * @return the onload event set
     */
    public function get_onload() {
        return $this->onload;
    }

    /**
     *
     * @param string $onnew_record
     * set the js on case of add and modify and place condition on add click and modify
     */
    public function set_onnew_record($onnew_record) {
        $this->onnew_record .= $onnew_record;
        return $this;
    }

    public function override_onnew_record($onnew_record) {
        $this->onnew_record = $onnew_record;
        return $this;
    }

    /**
     *
     * @return the onnew_record event set
     */
    public function get_onnew_record() {
        return $this->onnew_record;
    }

    /**
     *
     * @param string $onadd
     * set the js on case of add and modify and place condition on add click and modify
     */
    public function set_onadd($onadd) {
        $this->onadd = $onadd;
        return $this;
    }

    /**
     *
     * @return the onadd event set
     */
    public function get_onadd() {
        return $this->onadd;
    }

    /**
     *
     * @param string $onmodify
     * set the js on case of add and modify and place condition on add click and modify
     */
    public function set_onmodify($onmodify) {
        $this->onmodify = $onmodify;
        return $this;
    }

    /**
     *
     * @return the onmodify event set
     */
    public function get_onmodify() {
        return $this->onmodify;
    }

    /**
     * @param string $form_display
     * set dom form_display
     */
    public function set_form_display($form_display = "none") {
        $this->form_display = $form_display;
        return $this;
    }

    /**
     * @method public
     * @return string $form_display of dom object
     */
    public function get_form_display() {
        return $this->form_display;
    }

    /**
     * new function for grid
     */

    /**
     *
     * @param string $modifyTitle
     * @return $this
     */
    public function set_modify_title($modify_title = LANG_MODIFY) {
        $this->options['modifyTitle'] = $modify_title;
        return $this;
    }

    /**
     *
     * @return $modify_title
     */
    public function get_modify_title() {
        return $this->options['modifyTitle'];
    }

    /**
     *
     * @param string $deleteTitle
     * @return $this
     */
    public function set_delete_title($delete_title = LANG_DELETE) {
        $this->options['deleteTitle'] = $delete_title;
        return $this;
    }

    /**
     *
     * @return $delete_title
     */
    public function get_delete_title() {
        return $this->options['deleteTitle'];
    }

    /**
     *
     * @param string $printTitle
     * @return $this
     */
    public function set_print_title($print_title = LANG_PRINT) {
        $this->options['printTitle'] = $print_title;
        return $this;
    }

    /**
     *
     * @return $print_title
     */
    public function get_print_title() {
        return $this->options['printTitle'];
    }

    /**
     *
     * @param string $copyTitle
     * @return $this
     */
    public function set_copy_title($copy_title = LANG_COPY) {
        $this->options['copyTitle'] = $copy_title;
        return $this;
    }

    /**
     *
     * @return $copy_title
     */
    public function get_copy_title() {
        return $this->options['copyTitle'];
    }

    /**
     *
     * @param string $searchlable
     * @return $this
     */
    public function set_delete_confirmation_msg($delete_confirmation_msg = LANG_DOM_DELETE_MSG) {
        $this->options['deleteConfirmationMsg'] = $delete_confirmation_msg;
        return $this;
    }

    /**
     *
     * @return $delete_confirmation_msg
     */
    public function get_delete_confirmation_msg() {
        return $this->options['deleteConfirmationMsg'];
    }

    /**
     *
     * @param bool $editing_toolbar
     * @return $this
     */
    public function disable_context_menu($disable_context_menu = true) {

        $this->options['disableContextMenu'] = $disable_context_menu;

        return $this;
    }

    /**
     *
     * @return $editing_toolbar
     */
    public function get_disable_context_menu() {
        return $this->options['disableContextMenu'];
    }

    /**
     * set columns min width
     * @access public
     * @param int $min_col_width
     * @return none
     */
    public function set_min_col_width($min_col_width) {
        $this->options['minColWidth'] = $min_col_width;
        return $this;
    }

    /**
     *
     * @return minColWidth
     */
    public function get_min_col_width() {

        return $this->options['minColWidth'];
    }

    /**
     * set grid dir
     * @access public
     * @param int $hieght
     * @return none;
     */
    public function set_dir($dir = dir) {
        $this->options['dir'] = $dir;
        return $this;
    }

    /**
     * get grid dir
     * @param none
     * @return $dir => grid dir
     */
    public function get_dir() {
        $this->options['dir'];
    }

    /**
     * set grid height
     * @access public
     * @param int $hieght
     * @return none;
     */
    public function set_height($height) {
        $this->options['height'] = $height;
        return $this;
    }

    /**
     * get grid height
     * @param none
     * @return $height => grid height
     */
    public function get_height() {
        $this->options['height'];
    }

    /**
     * set grid form_height
     * @access public
     * @param int $hieght
     * @return none;
     */
    public function set_form_height($form_height) {
        $this->options['form_height'] = $form_height;
        return $this;
    }

    /**
     * get grid form_height
     * @param none
     * @return $form_height => grid form_height
     */
    public function get_form_height() {
        $this->options['form_height'];
    }

    /**
     * set grid width
     * @access public
     * @param int $hieght
     * @return none;
     */
    public function set_width($width) {
        $this->options['width'] = $width;
        return $this;
    }

    /**
     * get grid width
     * @param none
     * @return $width => grid width
     */
    public function get_width() {
        $this->options['width'];
    }

    /**
     * set grid form_position
     * @access public
     * @param int $hieght
     * @return none;
     */
    public function set_form_position($form_position = "absolute") {
        $this->options['formPosition'] = $form_position;
        return $this;
    }

    /**
     * get grid form_position
     * @param none
     * @return $form_position => grid form_position
     */
    public function get_form_position() {
        $this->options['formPosition'];
    }

    /**
     * set grid form_overflow_y
     * @access public
     * @param int $hieght
     * @return none;
     */
    public function set_form_overflow_y($form_overflow_y = "auto") {
        $this->options['formOverflowY'] = $form_overflow_y;
        return $this;
    }

    /**
     * get grid form_overflow_y
     * @param none
     * @return $form_overflow_y => grid form_overflow_y
     */
    public function get_form_overflow_y() {
        $this->options['formOverflowY'];
    }

    /**
     * set grid form_overflow_x
     * @access public
     * @param int $hieght
     * @return none;
     */
    public function set_form_overflow_x($form_overflow_x = "auto") {
        $this->options['formOverflowX'] = $form_overflow_x;
        return $this;
    }

    /**
     * get grid form_overflow_x
     * @param none
     * @return $form_overflow_x => grid form_overflow_x
     */
    public function get_form_overflow_x() {
        $this->options['formOverflowX'];
    }

    /**
     * set header height
     * @access public
     * @param int $header_height
     * @return none
     */
    public function set_header_height($header_height) {
        $this->options['headerHeight'] = $header_height;
        return $this;
    }

    /**
     *
     * @return bool headerHeight
     */
    public function get_header_height() {
        return $this->options['headerHeight'];
    }

    /**
     * defines the grid can be resized
     * @access public
     * @param bool set_is_resizable_cols
     * @return none
     */
    public function set_is_resizable_cols($is_resizable_cols = true) {

        $this->options['resizableCols'] = $is_resizable_cols;
        return $this;
    }

    /**
     *
     * @return bool resizableCols
     */
    public function get_is_resizable_cols() {

        return $this->options['resizableCols'];
    }

    /**
     * defines the row selection feature
     * @access public
     * @param bool $is_row_selection
     * @return none
     */
    public function set_is_row_selection($is_row_selection = true) {

        $this->options['rowSelection'] = $is_row_selection;
        return $this;
    }

    /**
     *
     * @return bool rowSelection
     */
    public function get_is_row_selection() {
        return $this->options['rowSelection'];
    }

    /**
     * defines extended_function feature
     * @access public
     * @return none
     */
    public function set_extended_function($extended_function = null) {

        $this->options['extendedFunction'] = $extended_function;
        return $this;
    }

    /**
     * defines set_onbefore_delete feature
     * @access public
     * @return none
     */
    public function add_before_delete($before_delete = null) {
        $this->before_delete[] = $before_delete;
        return $this;
    }

    /**
     * defines set_onbefore_delete feature
     * @access public
     * @return none
     */
    public function override_before_delete($before_delete = null) {
        unset($this->before_delete);
        $this->before_delete = array();
        $this->before_delete[0] = $before_delete;
        return $this;
    }

    /**
     * defines ondelete feature
     * @access public
     * @return none
     */
    public function override_ondelete($ondelete = null) {
        $this->ondelete = $ondelete;
        return $this;
    }

    /**
     * defines set_onafter_delete feature
     * @access public
     * @return none
     */
    public function add_after_delete($after_delete = null) {
        $this->after_delete[] = $after_delete;
        return $this;
    }

    /**
     * defines set_onafter_delete feature
     * @access public
     * @return none
     */
    public function override_after_delete($after_delete = null) {
        unset($this->after_delete);
        $this->after_delete = array();
        $this->after_delete[0] = $after_delete;
        return $this;
    }

    /**
     * defines set_onafter_add feature
     * @access public
     * @return none
     */
    public function add_after_add($after_add = null) {
        $this->after_add[] = $after_add;
        return $this;
    }

    /**
     * defines set_onafter_add feature
     * @access public
     * @return none
     */
    public function override_after_add($after_add = null) {
        unset($this->after_add);
        $this->after_add = array();
        $this->after_add[0] = $after_add;
        return $this;
    }

    /**
     *
     * @return
     */
    public function get_after_add() {
        return implode("; ", $this->after_add);
    }

    /**
     * get ondelete
     */
    protected function get_on_delete() {
        $this->before_delete[] = $this->ondelete;
        $this->options['onDelete'] = implode("; ", array_merge($this->before_delete, $this->after_delete));
    }

    /**
     * defines set_onbefore_show_modify_form feature
     * @access public
     * @return none
     */
    public function add_before_show_modify_form($before_show_modify_form = null) {
        $this->before_show_modify_form[] = $before_show_modify_form;
        return $this;
    }

    /**
     * defines set_onbefore_show_modify_form feature
     * @access public
     * @return none
     */
    public function override_before_show_modify_form($before_show_modify_form = null) {
        unset($this->before_show_modify_form);
        $this->before_show_modify_form = array();
        $this->before_show_modify_form[0] = $before_show_modify_form;
        return $this;
    }

    /**
     * defines onshow_modify_form feature
     * @access public
     * @return none
     */
    public function override_onshow_modify_form($onshow_modify_form = null) {
        $this->onshow_modify_form = $onshow_modify_form;
        return $this;
    }

    /**
     * defines set_onafter_show_modify_form feature
     * @access public
     * @return none
     */
    public function add_after_show_modify_form($after_show_modify_form = null) {
        $this->after_show_modify_form[] = $after_show_modify_form;
        return $this;
    }

    /**
     * defines set_onafter_show_modify_form feature
     * @access public
     * @return none
     */
    public function override_after_show_modify_form($after_show_modify_form = null) {
        unset($this->after_show_modify_form);
        $this->after_show_modify_form = array();
        $this->after_show_modify_form[0] = $after_show_modify_form;
        return $this;
    }

    /**
     * get onshow_modify_form
     */
    protected function get_on_show_modify_form() {
        $this->before_show_modify_form[] = $this->onshow_modify_form;
        $this->options['onShowModifyForm'] = implode("; ", array_merge($this->before_show_modify_form, $this->after_show_modify_form));
    }

    /**
     * defines set_onbefore_copy feature
     * @access public
     * @return none
     */
    public function add_before_copy($before_copy = null) {
        $this->before_copy[] = $before_copy;
        return $this;
    }

    /**
     * defines set_onbefore_copy feature
     * @access public
     * @return none
     */
    public function override_before_copy($before_copy = null) {
        unset($this->before_copy);
        $this->before_copy = array();
        $this->before_copy[0] = $before_copy;
        return $this;
    }

    /**
     * defines oncopy feature
     * @access public
     * @return none
     */
    public function override_oncopy($oncopy = null) {
        $this->oncopy = $oncopy;
        return $this;
    }

    /**
     * defines set_onafter_copy feature
     * @access public
     * @return none
     */
    public function add_after_copy($after_copy = null) {
        $this->after_copy[] = $after_copy;
        return $this;
    }

    /**
     * defines set_onafter_copy feature
     * @access public
     * @return none
     */
    public function override_after_copy($after_copy = null) {
        unset($this->after_copy);
        $this->after_copy = array();
        $this->after_copy[0] = $after_copy;
        return $this;
    }

    /**
     * get oncopy
     */
    protected function get_on_copy() {
        $this->before_copy[] = $this->oncopy;
        $this->options['onCopy'] = implode("; ", array_merge($this->before_copy, $this->after_copy));
    }

    /**
     *
     * @param string $title
     * @param string $obj
     * @param string $alias
     * create the dom structure
     */
    public function new_element($obj, $title = null, $alias = null, $width = null, $align = DIR_ALIGN, $header_align = DIR_ALIGN) {

        //set the constant field of the element
        if ($title)
            $constant = $title;
        else
        if (@constant($obj->get_name()))
            $constant = @constant($obj->get_name());
        else
            $constant = LANG_DOM_NO_TITLE;
        if (!$width)
            $width = $this->get_min_col_width();
        //create the form display of dom object
        $this->options['colWidths'][] = $width;
        $this->options['colAligns'][] = $align;
        $this->options['headerAligns'][] = $header_align;

        parent::new_element('', '<div>' . $constant . '</div>' . $obj->load(), 1);

        switch (get_class($obj)) {
            case 'combo':
                $this->label_element[$obj->get_name()] = $obj->get_name() . '_label';
                $this->set_onnew_record(" {$obj->get_name()}.setComboValue('{$obj->get_value()}',false); ");
                break;
            case 'calendar':
                $this->set_onnew_record(" $('#{$obj->get_id()}').date_val('{$obj->get_value()}'); ");
                break;
            case 'input_money':
                //$this->label_element[$obj->get_name()]=$obj->get_name().'_label';
                $this->set_onnew_record(" {$obj->get_name()}.val('{$obj->get_value()}');");
                break;
            default:
                $this->set_onnew_record(" $('#{$obj->get_id()}').val('{$obj->get_value()}'); ");
                break;
        }

        //html
        $this->element[$obj->get_name()] = $obj; //create the array for html drawing of hidden fields for cloning
        $this->header .= '<th>' . $constant . '</th>'; //create the header
        //select stament fields
        $this->select_field .= $this->get_db_table_name() . '.' . $obj->get_name() . ','; //normal select fro the table set
        if ($alias) //to allow the select of fields that are related to other table in case of standard_id you need to see the standard name
            $this->select_field .= $alias . ' as ' . $obj->get_name() . '_alias' . ','; //if alias exist

            
// if alias is set than we have to select the alias field to allow html label of combo or date.
        $this->alias_element[$obj->get_name()] = $obj->get_name();
        if ($alias)
            $this->alias_element[$obj->get_name()] = $obj->get_name() . '_alias';

        //insert to db fields
        array_push($this->insert_field, $obj->get_name());
        return $this;
    }

    /**
     *
     * @param  $obj
     * @return $this
     */
    public function add_hidden_input($name, $value = null) {
        $this->hidden_inputs[$name] = $value;
        array_push($this->insert_field, $name);
        $this->set_onnew_record("$('#{$name}').val('{$value}'); ");
        return $this;
    }

    public function get_hidden_inputs() {
        return $this->hidden_inputs;
    }

    /**
     *
     * @param int $parent_id
     */
    public function add($parent_id = null, $reset_post = true) {

        if ($parent_id) //set the parent id in case of add form where we get the last insert id
            $_POST[$this->get_parent_id()] = $parent_id;

        $this->delete(); //delete or previous rows from table of the parent table of the id set

        $counter = 0; //to return how many rows will be set
        foreach ($this->get_insert_field() as $key => $value) {
            $counter = count($_POST[$value]);
            break;
        }

        //start looping at one so the hidden fields of the set element arent included
        for ($i = 1; $i < $counter; $i++) {
            //building the structure of insert statment
            $buildset = "`" . $this->get_parent_id() . "`=" . fix_value($_POST[$this->get_parent_id()]);
            //loop on the insert field array to build the structure of the fields
            foreach ($this->get_insert_field() as $key => $value) {
                if ($_POST[$value][$i])
                    $buildset .= ",`$value`=" . fix_value($_POST[$value][$i]);
                if ($reset_post)
                    unset($_POST[$value][$i]);
            }
            /**
             * we stop below since we make "array_push($this->insert_field,$name);"
             * in the add_hidden_input function
              foreach($this->get_hidden_inputs() as $name=>$value) {
              $buildset.=",`$name`=".fix_value($_POST[$name][$i]);
              if($reset_post) unset($_POST[$name][$i]);
              }
             */
            //set the statment for insertion
            $statment = "insert into `" . $this->get_db_table_name() . "` set " . $buildset;
            $this->sql->execute_statment($statment, $this->get_debug());
        }
        return $this;
    }

    /**
     *
     * @param int $parent_id
     * modify action
     */
    public function modify($parent_id = null) {
        $this->add($parent_id, false);
        return $this;
    }

    /**
     * delete statment
     */
    public function delete($parent_id = null) {
        if ($parent_id)
            $_REQUEST[$this->get_parent_id()] = $parent_id;

        $statment = "delete from  `" . $this->get_db_table_name() . "`
                          where `" . $this->get_parent_id() . "`=" . fix_value($_REQUEST[$this->get_parent_id()]);
        $this->sql->execute_statment($statment, $this->get_debug());
        return $this;
    }

    /**
     * @return string
     * build the dom on modify form
     */
    protected function get_modify_dom() {
        $hidden_fields = null;
        if ($this->get_hidden_inputs())
            $hidden_fields = "," . implode(",", array_keys($this->get_hidden_inputs()));

        if ($_REQUEST[$this->get_parent_id()]) {
            //build the statment
            $statment = "select " . $this->get_select_field() . $hidden_fields . " from `" . $this->get_db_table_name() . "` ";
            $statment .= $this->get_join();
            $statment .= " where `" . $this->get_db_table_name() . "`." . $this->get_parent_id() . "=" . fix_value($_REQUEST[$this->get_parent_id()]);
            if ($this->get_order())
                $statment .= " order by " . $this->get_order();
            $result = $this->sql->get_result($statment, $this->get_debug());
            $rows = array();
            if ($result->num_rows > 0) {
                while ($row = $this->sql->next($result)) {
                    //build the dom body element
                    $rows[] = $this->get_body_dom($row);
                }
            }
        }

        if ($rows)
            $str = implode("", $rows);


        return $str;
    }

    /**
     *
     * @param array $data
     * @return string
     * build the hidden fields and the body in case of modify if data is set
     */
    protected function get_body_dom($data = null) {

        $columns = array();
        foreach ($this->get_element() as $key => $value) {

            $label = $this->set_label_value($data[$this->get_alias_element($key)], $value);
            $val = $data[$key];
            $val = $this->set_input_value($data[$key], $value);


            $element = '<input type="hidden"  name="' . $key . '[]" id="' . $key . '" value="' . $val . '">';
            if ($this->label_element[$key]) {
                $label_element = $this->label_element[$key];
                $element .= '<input type="hidden"  name="' . $label_element . '[]" id="' . $label_element . '" value="' . $label . '">';
            }
            $columns[] = "<label class='nowrap_div'>{$label}</label>{$element}";
        }
        if ($this->get_hidden_inputs()) {
            $hidden_elements = array();
            foreach ($this->get_hidden_inputs() as $name => $value) {
                if ($data)
                    $value = $data[$name];

                $hidden_elements[] = "<input type='hidden'  name='{$name}[]' id='{$name}' value='{$value}'>";
            }
        }

        if ($columns)
            $str = "<tr><td>" . implode("</td><td>", $columns);

        if ($this->get_hidden_inputs())
            $str .= implode("", $hidden_elements);
        $str .= "</td></tr>";
        return $str;
    }

    protected function get_data_dom() {
        $counter = 0; //to return how many rows will be set
        foreach ($this->get_insert_field() as $key => $value) {
            $counter += count($_POST[$value]);
            break;
        }
        if ($counter > 0) {
            for ($i = 1; $i < $counter; $i++) {

                //building the structure of insert statment
                $columns = array();
                foreach ($this->get_element() as $key => $value) {
                    $val = $_POST[$key][$i];
                    if ($_POST[$key . '_label'][$i])
                        $label = $_POST[$key . '_label'][$i];
                    else
                        $label = $this->set_label_value($val, $value);

                    $element = '<input type="hidden"  name="' . $key . '[]" id="' . $key . '" value="' . $val . '">';
                    if ($this->label_element[$key]) {
                        $label_element = $this->label_element[$key];
                        $element .= '<input type="hidden"  name="' . $label_element . '[]" id="' . $label_element . '" value="' . $label . '">';
                    }
                    $columns[] = "<label class='nowrap_div'>{$label}</label>{$element}";
                }
                $hidden_elements = array();
                if ($this->get_hidden_inputs()) {
                    foreach ($this->get_hidden_inputs() as $name => $value) {
                        $val = $_POST[$name][$i];

                        $hidden_elements[] = "<input type='hidden'  name='{$name}[]' id='{$name}' value='{$val}'>";
                    }
                }
                $str .= "<tr><td>" . implode("</td><td>", $columns);

                if ($this->get_hidden_inputs())
                    $str .= implode("", $hidden_elements);

                $str .= "</td></tr>";
            }
        } else
            $str = $this->get_modify_dom();

        return $str;
    }

    /**
     *
     * @param  string|date|int $value
     * @return value
     */
    protected function set_label_value($value, $obj) {
        //set the label value
        $class_name = get_class($obj);
        if ($class_name == 'calendar' && $value) {
            if ($value == '0000-00-00')
                return $value = '';

            return date($obj->get_date_format(), strtotime($value));
        }
        if ($class_name == 'input_money') {
            if ($value > 0)
                return number_format($value, $obj->get_precision(), '.', ',');

            if ($value == 0 && $obj->get_enable_zero_value() == false)
                return '';

            return number_format(0, $obj->get_precision(), '.', ',');
        }
        if ($class_name == 'combo' && !$value)
            return "";

        return $value;
    }

    protected function set_input_value($value, $obj) {
        //set the label value
        $class_name = get_class($obj);
        if ($class_name == 'calendar') {
            if ($value == '0000-00-00')
                return $value = '';
        }
        if ($class_name == 'input_money') {
            if ($value == 0 && $obj->get_enable_zero_value() == false)
                return '';
        }
        return $value;
    }

    /**
     *
     * @return th header set at the new_element
     */
    protected function get_header_dom() {
        return $this->header;
    }

    public function set_readonly($bool = true) {
        $this->disable_context_menu($bool);
        return $this;
    }

    public function get_buttons_container() {
        return $this->get_name() . "_clone_filter_button_div";
    }

    public function get_form_container() {
        return $this->get_name() . "_clone_filter_form_div";
    }

    /**
     *
     * @return string
     *
     */
    protected function set_buttons() {


        $str = '<table cellpadding="2" cellspacing="2" align="center" dir="' . dir . '">
                 <tr>';

        if ($this->get_allow_add())
            $str .= '<td><button type="button" name="add" id="' . $this->get_name() . '_add">' . LANG_ADD . '</button></td>';

        if ($this->get_allow_modify())
            $str .= '<td><button type="button" name="modify" id="' . $this->get_name() . '_modify">' . LANG_MODIFY . '</button></td>';

        if ($this->get_allow_new())
            $str .= '<td><button type="button" id="' . $this->get_name() . '_new_record">' . LANG_NEW_RECORD . '</button></td>';
        //create the buttons of the dom grid in order to control the add |modify|delete
        if ($this->get_button()) {
            foreach ($this->get_button() as $key => $value) {
                $str .= '<td>' . $value . '</td>';
            }
        }

//        $str.='     <td><input type="button" value="'.LANG_HIDE.'"  id="'.$this->get_name().'_show_hide" ></td>';
        $str .= '  </tr>
              </table>';
        return $str;
    }

    /**
     *
     * @return <string
     */
    protected function set_dom() {

        if ($this->get_element()) {
            $str = ' <table id="' . $this->get_name() . '_hidden" style="display:none;">';
            $str .= $this->get_body_dom();
            $str .= '</table>';


            $str .= ' <table id="' . $this->get_name() . '_clone">
                            <thead>
                                <tr>' . $this->get_header_dom() . '
                                </tr>
                             </thead>
                            <tbody>' . $this->get_data_dom() . '</tbody>
                         </table>';
        }
        return $str;
    }

    /**
     *
     * @return string
     * echo the dom object
     */
    public function load() {


        foreach ($this->get_hidden_inputs() as $name => $value) {
            $obj = new input_hidden($name);
            $obj->set_value($value);
            $row_html .= $obj->load();
        }

        parent::hidden_row($row_html);
        $load = '<div id="' . $this->get_name() . '" class="dom">';
        if (!$this->get_disable_context_menu()) {
            $load .= '<div id="' . $this->get_name() . '_buttons" class="dom_table_buttons">';
            if ($this->get_allow_modify())
                $load .= '<div class="dom-button" id="' . $this->get_name() . '_modify"><i class="mdi mdi-floppy"></i></div>';
            if ($this->get_allow_add())
                $load .= '<div class="dom-button" id="' . $this->get_name() . '_add"><i class="mdi mdi-keyboard-return"></i></div>';
            if ($this->get_allow_new())
                $load .= '<div class="dom-button" id="' . $this->get_name() . '_new_record"><i class="mdi mdi-close-circle-outline"></i></div>';

            $load .= '</div>';

            $load .= '<div id="' . $this->get_name() . '_clone_filter_div" style="display:block;width:' . $this->options['width'] . 'px;overflow:auto">
                  <table><tr><td>' . parent::load() . '</td><td><div style="width:90px;"></div></td></tr></table> 
                </div>';
        }
        $load .= $this->set_dom();
        $this->options['gridType'] = "dom";
        $this->options['sorting'] = false;
        $this->options['paging'] = false;
        $this->options['resizableCols'] = true;

        $this->get_on_delete();
        $this->get_on_copy();
        $this->get_on_show_modify_form();
        $auto_hide_form = null;
        $load .= '</div>
            <script type="text/javascript">
            var ' . $this->get_name() . '_grid;
            $(document).ready(
                function() {
                    var dom,grid,dom_hidden,dom_clone;';
        $load .= $this->get_name() . '_grid = $("#' . $this->get_name() . '_clone").ingrid(' . json_encode($this->options) . ');
                         ' . $this->get_name() . '_grid.g.initStylesAndWidths();
                          dom   = ' . $this->get_name() . '_grid.dom;
                          grid  = ' . $this->get_name() . '_grid.g;
                          dom_hidden = $("#' . $this->get_name() . '_hidden");
                          dom_clone  = $("#' . $this->get_name() . '_clone");
                          dom.newRecordAction();
                         ';

        if ($this->get_onadd() AND $this->get_allow_add())
            $load .= '    $("#' . $this->get_name() . '_add").bind("click",function(){
                            ' . $this->get_onadd() . '
                         });';
        if ($this->get_onmodify() AND $this->get_allow_modify())
            $load .= '    $("#' . $this->get_name() . '_modify").bind("click",function(){
                            ' . $this->get_onmodify() . '
                         });';

        $load .= '    $("#' . $this->get_name() . '_new_record").click(function(){
                            dom.newRecordAction();
                            ' . $this->get_onnew_record() . '
                         });';

        if ($this->get_allow_add())
            $load .= '    $("#' . $this->get_name() . '_add").bind("click",function(){
                            ' . $this->get_onchange() . '
                            dom.addAction();
                            ' . $this->get_name() . '_grid.g.initRowStylesAndWidths();
                            ' . $this->get_after_add() . '
                            ' . $auto_hide_form . ';
                         });';
        if ($this->get_allow_modify()) {
            $load .= '     $("#' . $this->get_name() . '_modify").bind("click",function(){
                            ' . $this->get_onchange() . '
                            dom.modifyAction();
                            ' . $this->get_name() . '_grid.g.initRowStylesAndWidths();';
            $load .= '
                });';
        }
        $load .= '     $("#' . $this->get_name() . '_modify,#' . $this->get_name() . '_add").bind("click",function(event){
                                event.preventDefault();
                                return false;
                            });';
        $load .= $this->get_onload() . '
            ';
        $load .= $this->get_script() . '
            ';
        $load .= '

                       })';
        $load .= '</script>';

        return $load;
    }

}

?>