<?php

/**
 * grid parser class . use this class to create search grid and its filtration
 * inherited from form_parser class
 * @author Ihab A. Ali <ihab.a.ali@gmail.com>
 * @copyright copyright (c) copyright 2020
 * @license GNU Public Licence (GPL)
 * @example  $grid = new grid($url)
 */
class grid {

    /**
     *
     * @var $name
     */
    private $name = null;

    /**
     * it will be use to set all the needed attr to initiate grid
     * @access private
     * @var array $attr
     */
    private $attr = array();

    /**
     * this will be used later on to set the header titles
     * inside an array
     * @access private
     * @var array $header_titles
     */
    private $header_titles = array();

    /**
     * this will be used later on to set the header db columns
     * inside an array
     * @access private
     * @var array $header_columns
     */
    private $header_columns = array();

    /**
     * set the form html code
     * @access private
     * @var string $filte_form
     */
    private $filte_form = null;

    /**
     * string to store the filteration parameters
     * @var string $params
     */
    private $params = array();

    /**
     * array to store extra parameters
     * @var array $extra_params
     */
    private $extra_params = array();

    /**
     *
     * @var string $modify_iframe
     */
    private $modify_iframe = 'tab_iframe0';
    
    private $request_on_load = false;
    /**
     * @param string $db_table_name
     * @param string $url
     */
    public function __construct($name = null) {
        if (!$name)
            $this->set_name(preg_replace("/.php/", "", self_name));
        else
            $this->set_name($name);
        $this->set_min_col_width(80)
                ->set_is_resizable_cols()
                ->set_height(350)
                ->set_dir(dir)
                ->set_url(self)
                ->set_records_per_page(50)
                ->set_type('POST')
                ->set_search_action('search')
                ->set_initial_load(false)
                ->show_export_Xls()
                ->set_modify_title()
                ->set_copy_title()
                ->set_delete_title()
                ->set_print_title()
                ->set_delete_confirmation_msg()
                ->set_group_delete_confirmation_msg()
                ->set_no_rows_selected_err()
                ->set_editing_target()
                ->disable_group_delete();
        //just to overwrite it & set auto, if not set in form module
        $this->set_width('auto')->set_height('auto');
        
//        $this->attr['records'] = LANG_SEARCH;
        $this->attr['of'] = "'".LANG_OUT_OF."'";
        $this->attr['no_result'] = "'".LANG_NO_RESULT_MSG."'";
        $this->attr['no_connection_err'] = "'".LANG_NO_CONNECTION_ERR."'";
        $this->attr['page'] = "'".LANG_GRID_PAGE."'";
        $this->attr['searchButtonTitle'] = "'".LANG_GRID_ADV_SEARCH_TITLE."'";
        $this->attr['pagePrevTitle'] = "'".LANG_GRID_NAV_PREV."'";
        $this->attr['pageNextTitle'] = "'".LANG_GRID_NAV_NEXT."'";
        $this->attr['exportXlsTitle'] = "'".LANG_EXPORT_XLS."'";
        $this->attr['reloadButtonTitle'] = "'".LANG_RELOAD."'";
        $this->attr['quickSearchTitle'] = "'".LANG_GRID_SEARCH_INPUT_TIP."'";
        $this->attr['searchInputPlaceholder'] = "'".LANG_SEARCH."'";
        $this->attr['totalRecordsTitle'] = "'".LANG_GRID_TOTAL_RECORDS_TITLE."'";
//        $this->attr['groupDeleteBtTitle'] = LANG_SEARCH;
//        $this->attr['valid_number_msg'] = LANG_SEARCH;


    }

    /**
     *
     * @param string $name
     */
    public function set_name($name = 'table1') {
        $this->name = $name;
        return $this;
    }

    /**
     *
     * @return $this->name
     */
    public function get_name() {
        return $this->name;
    }
    /**
     *
     * @param string $request_on_load
     */
    public function request_on_load($request_on_load = true) {
        $this->request_on_load = $request_on_load;
        return $this;
    }

    /**
     *
     * @return $this->request_on_load
     */
    public function get_request_on_load() {
        return $this->request_on_load;
    }

    /**
     * set grid showFilter
     * @access public
     * @param bool showFilter
     * @return none;
     */
    public function debug($enable_debug = true) {
        if ($enable_debug == true)
            $this->attr['enableDebug'] = "true";
        else
            $this->attr['enableDebug'] = "false";
        return $this;
    }

    /**
     * get enableDebug
     * @param none
     * @return bool $enableDebug
     */
    public function get_enable_debug() {
        if ($this->attr['enableDebug'] == 'true')
            return true;
        else
            return false;
    }

    /**
     * set grid showExportXls
     * @access public
     * @param bool showExportXls
     * @return none;
     */
    public function show_export_xls($bool = true) {
        if ($bool == true)
            $this->attr['showExportXls'] = "true";
        else
            $this->attr['showExportXls'] = "false";
        return $this;
    }

    /**
     * get showExportXls
     * @param none
     * @return bool $showExportXls
     */
    public function get_show_export_xls() {
        if ($this->attr['showExportXls'] == 'true')
            return true;
        else
            return false;
    }

    /**
     * set grid showFilter
     * @access public
     * @param bool showFilter
     * @return none;
     */
    public function set_show_filter($showFilter = true) {
        if ($showFilter == true)
            $this->attr['showFilter'] = "true";
        else
            $this->attr['showFilter'] = "false";
        return $this;
    }

    /**
     * get showFilter
     * @param none
     * @return bool $showFilter
     */
    public function get_show_filter() {
        if ($this->attr['showFilter'] == 'true')
            return true;
        else
            return false;
    }

    /**
     * defines extended_function feature
     * @access public
     * @return none
     */
    public function set_extended_function($extended_function = null) {

        $this->attr['extendedFunction'] = json_encode($extended_function);
        return $this;
    }

    /**
     * set grid dir
     * @access public
     * @param int $hieght
     * @return none;
     */
    public function set_dir($dir) {
        $this->attr['dir'] = json_encode($dir);
        return $this;
    }

    /**
     * get grid dir
     * @param none
     * @return $dir => grid dir
     */
    public function get_dir() {
        $this->attr['dir'];
    }

    /**
     * set grid height
     * @access public
     * @param int $hieght
     * @return none;
     */
    public function set_height($height) {
        $height = 'auto';
        $this->attr['height'] = json_encode($height);
        return $this;
    }

    /**
     * get grid height
     * @param none
     * @return $height => grid height
     */
    public function get_height() {
        return $this->attr['height'];
    }

    /**
     * set grid width
     * @access public
     * @param int $hieght
     * @return none;
     */
    public function set_width($width) {
        $width = 'auto';
        $this->attr['width'] = json_encode($width);
        return $this;
    }

    /**
     * get grid width
     * @param none
     * @return $width => grid width
     */
    public function get_width() {
        return $this->attr['width'];
    }

    /**
     * set header height
     * @access public
     * @param int $header_height
     * @return none
     */
    public function set_header_height($header_height) {
        $this->attr['headerHeight'] = $header_height;
        return $this;
    }

    /**
     *
     * @return bool headerHeight
     */
    public function get_header_height() {
        return $this->attr['headerHeight'];
    }

    /**
     * defines the grid has sorting feature
     * @access public
     * @param bool $is_sorting
     * @return none
     */
    public function set_is_sorting($is_sorting = true) {
        if ($is_sorting == true)
            $this->attr['sorting'] = "true";
        else
            $this->attr['sorting'] = "false";
        return $this;
    }

    /**
     *
     * @return bool sorting
     */
    public function get_is_sorting() {
        if ($this->attr['sorting'] == 'true')
            return true;
        else
            return false;
    }

    /**
     * defines the grid can be resized
     * @access public
     * @param bool set_is_resizable_cols
     * @return none
     */
    public function set_is_resizable_cols($is_resizable_cols = true) {
        if ($is_resizable_cols == true)
            $this->attr['resizableCols'] = "true";
        else
            $this->attr['resizableCols'] = "false";
        return $this;
    }

    /**
     *
     * @return bool resizableCols
     */
    public function get_is_resizable_cols() {
        if ($this->attr['resizableCols'] == 'true')
            return true;
        else
            return false;
    }

    /**
     * defines the row selection feature
     * @access public
     * @param bool $is_row_selection
     * @return none
     */
    public function set_is_row_selection($is_row_selection = true) {
        if ($is_row_selection == true)
            $this->attr['rowSelection'] = "true";
        else
            $this->attr['rowSelection'] = "false";
        return $this;
    }

    /**
     *
     * @return bool rowSelection
     */
    public function get_is_row_selection() {
        if ($this->attr['rowSelection'] == 'true')
            return true;
        else
            return false;
    }

    /**
     * defines the row paging feature
     * @access public
     * @param bool set_is_paging
     * @return none
     */
    public function set_is_paging($is_paging = true) {
        if ($is_paging == true)
            $this->attr['paging'] = "true";
        else
            $this->attr['paging'] = "false";

        return $this;
    }

    /**
     *
     * @return bool $is_paging
     */
    public function get_is_paging() {
        if ($this->attr['paging'] == 'true')
            return true;
        else
            return false;
    }

    /**
     * defines onload page number
     * @access public
     * @param int $initial_page_no
     * @return none
     */
    public function set_initial_page_no($initial_page_no) {
        $this->attr['pageNumber'] = $initial_page_no;
        return $this;
    }

    /**
     *
     * @return int pageNumber
     */
    public function get_initial_page_no() {
        return $this->attr['pageNumber'];
    }

    /**
     * defines the number of records per page
     * @access public
     * @param int $records_per_page
     * @return none
     */
    public function set_records_per_page($records_per_page) {
        $this->attr['recordsPerPage'] = $records_per_page;
        return $this;
    }

    /**
     *
     * @return int recordsPerPage
     */
    public function get_records_per_page() {
        return $this->attr['recordsPerPage'];
    }

    /**
     * set the remote file of the grid ajax . which  its the file that will execute the query and get the results
     * @access public
     * @param string $url
     * @return none
     */
    public function set_url($url = null) {
        $this->attr['url'] = json_encode($url);
        return $this;
    }

    /**
     *
     * @return string url
     */
    public function get_url() {
        return $this->attr['url'];
    }

    /**
     * send static vars to be added to grid actions links.
     * @access public
     * @param string $extend_actions
     * @return none
     */
    public function extend_actions($extend_actions = null) {
        $this->attr['extendActions'] = json_encode($extend_actions);
        return $this;
    }

    /**
     *
     * @return string extend_actions
     */
    public function get_extend_actions() {
        return $this->attr['extendActions'];
    }

    /**
     *
     * @param string $modifyTitle
     * @return $this
     */
    public function set_modify_title($modify_title = LANG_MODIFY) {
        $this->attr['modifyTitle'] = json_encode($modify_title);
        return $this;
    }

    /**
     *
     * @return $modify_title
     */
    public function get_modify_title() {
        return $this->attr['modifyTitle'];
    }

    /**
     *
     * @param string $deleteTitle
     * @return $this
     */
    public function set_delete_title($delete_title = LANG_DELETE) {
        $this->attr['deleteTitle'] = json_encode($delete_title);
        return $this;
    }

    /**
     *
     * @return $delete_title
     */
    public function get_delete_title() {
        return $this->attr['deleteTitle'];
    }

    /**
     *
     * @param string $printTitle
     * @return $this
     */
    public function set_print_title($print_title = LANG_PRINT) {
        $this->attr['printTitle'] = json_encode($print_title);
        return $this;
    }

    /**
     *
     * @return $print_title
     */
    public function get_print_title() {
        return $this->attr['printTitle'];
    }

    /**
     *
     * @param string $copyTitle
     * @return $this
     */
    public function set_copy_title($copy_title = LANG_COPY) {
        $this->attr['copyTitle'] = json_encode($copy_title);
        return $this;
    }

    /**
     *
     * @return $copy_title
     */
    public function get_copy_title() {
        return $this->attr['copyTitle'];
    }

    /**
     *
     * @param string $searchlable
     * @return $this
     */
    public function set_delete_confirmation_msg($delete_confirmation_msg = LANG_DOM_DELETE_MSG) {
        $this->attr['deleteConfirmationMsg'] = json_encode($delete_confirmation_msg);
        return $this;
    }

    /**
     *
     * @return $delete_confirmation_msg
     */
    public function get_delete_confirmation_msg() {
        return $this->attr['deleteConfirmationMsg'];
    }

    /**
     *
     * @param string groupDeleteConfirmationMsg
     * @return $this
     */
    public function set_group_delete_confirmation_msg($group_delete_confirmation_msg = LANG_GRID_GRP_DELETE_MSG) {
        $this->attr['groupDeleteConfirmationMsg'] = json_encode($group_delete_confirmation_msg);
        return $this;
    }

    /**
     *
     * @return $group_delete_confirmation_msg
     */
    public function get_group_delete_confirmation_msg() {
        return $this->attr['groupDeleteConfirmationMsg'];
    }

    /**
     *
     * @param string groupDeleteConfirmationMsg
     * @return $this
     */
    public function set_no_rows_selected_err($no_rows_selected_err = LANG_GRID_NO_ROWS_SELECTED_ERR) {
        $this->attr['noSelectedRowsErr'] = json_encode($no_rows_selected_err);
        return $this;
    }

    /**
     *
     * @return $no_rows_selected_err
     */
    public function get_no_rows_selected_err() {
        return $this->attr['noSelectedRowsErr'];
    }

    /**
     *
     * @param bool $editing_toolbar
     * @return $this
     */
    public function disable_modify($disable_modify = true) {
        if ($disable_modify == true)
            $this->attr['disableModify'] = "true";
        else
            $this->attr['disableModify'] = "false";

        return $this;
    }

    /**
     *
     * @return $editing_toolbar
     */
    public function get_disable_modify() {
        return $this->attr['disableModify'];
    }

    /**
     *
     * @param bool $editing_toolbar
     * @return $this
     */
    public function disable_copy($disable_copy = true) {
        if ($disable_copy == true)
            $this->attr['disableCopy'] = "true";
        else
            $this->attr['disableCopy'] = "false";

        return $this;
    }

    /**
     *
     * @return $editing_toolbar
     */
    public function get_disable_copy() {
        return $this->attr['disableCopy'];
    }

    /**
     *
     * @param bool $editing_toolbar
     * @return $this
     */
    public function disable_delete($disable_delete = true) {

        if ($disable_delete == true)
            $this->attr['disableDelete'] = "true";
        else
            $this->attr['disableDelete'] = "false";

        return $this;
    }

    /**
     *
     * @return $editing_toolbar
     */
    public function get_disable_delete() {
        return $this->attr['disableDelete'];
    }

    /**
     *
     * @param bool $enable_print
     * @return $this
     */
    public function enable_print($enable_print = true) {

        if ($enable_print == true)
            $this->attr['enablePrint'] = "true";
        else
            $this->attr['enablePrint'] = "false";
        return $this;
    }

    /**
     *
     * @return $enable_print
     */
    public function get_enable_print() {
        return $this->attr['enablePrint'];
    }

    /**
     *
     * @param bool $editing_toolbar
     * @return $this
     */
    public function disable_context_menu($disable_context_menu = true) {
        if ($disable_context_menu)
            $this->attr['disableContextMenu'] = "true";
        else
            $this->attr['disableContextMenu'] = "false";

        return $this;
    }

    /**
     *
     * @return $editing_toolbar
     */
    public function get_disable_context_menu() {
        return $this->attr['disableContextMenu'];
    }

    /**
     *
     * @param bool $editing_toolbar
     * @return $this
     */
    public function disable_group_delete($disable_group_delete = true) {
        if ($disable_group_delete)
            $this->attr['disableGroupDelete'] = "true";
        else
            $this->attr['disableGroupDelete'] = "false";

        return $this;
    }

    /**
     *
     * @return $editing_toolbar
     */
    public function get_disable_group_delete() {
        return $this->attr['showGroupDelete'];
    }

    /**
     * defines request method is post or get
     * @access public
     * @param string $type
     * @return none
     */
    public function set_type($type = 'POST') {
        $this->attr['type'] = json_encode($type);
        return $this;
    }

    /**
     *
     * @return string type <post or get>
     */
    public function get_type() {
        return $this->attr['type'];
    }

    /**
     * defines the location of modify tab target , in order to direct the record to its link
     * @access public
     * @param string $tab_location
     * @return none
     */
    public function set_targeted_tab($targeted_tab = null) {
        $this->attr['targetedTab'] = $targeted_tab;
        return $this;
    }

    /**
     * returns the location of modify form , in order to direct the record to its link
     * @access public
     * @return string $tab_location
     */
    public function get_targeted_tab() {
        return $this->attr['targetedTab'];
    }

    /**
     * defines the location of modify tab target , in order to direct the record to its link
     * @access public
     * @param string $tab_location
     * @return none
     */
    private function set_editing_target($editing_target = null) {
        $this->attr['editingTarget'] = json_encode($this->get_modify_iframe());
        return $this;
    }

    /**
     * returns the location of modify form , in order to direct the record to its link
     * @access public
     * @return string $tab_location
     */
    public function get_editing_target() {
        return $this->attr['editingTarget'];
    }

    /**
     * defines php action variable of search
     * @access public
     * @param string $action
     * @return none
     */
    public function set_search_action($search_action = null) {
        $this->attr['searchAction'] = '{action:"' . $search_action . '"}';
        return $this;
    }

    /**
     * returns the search action php variable
     * @access public
     * @return string $search_action
     */
    public function get_search_action() {
        return $this->attr['searchAction'];
    }

    /**
     * defines if the grid should load data on the page loading .
     * by default this feature is = disabled
     * @access public
     * @param bool $is_initial_load
     * @return none
     */
    public function set_initial_load($is_initial_load = true) {
        if ($is_initial_load == true)
            $this->attr['initialLoad'] = "true";
        else
            $this->attr['initialLoad'] = "false";
        return $this;
    }

    /**
     *
     * @return bool initialLoad
     */
    public function get_initial_load() {
        if ($this->attr['initialLoad'] == 'true')
            return true;
        else
            return false;
    }

    /**
     * defines if the grid should save its last states in the jquery cookie
     * by default this feature is = disabled
     * @access public
     * @param bool $is_saved_state_load
     * @return none
     */
    public function set_is_saved_state_load($is_saved_state_load = true) {
        if ($is_saved_state_load == true)
            $this->attr['savedStateLoad'] = "true";
        else
            $this->attr['savedStateLoad'] = "false";
        return $this;
    }

    /**
     *
     * @return bool savedStateLoad
     */
    public function get_is_saved_state_load() {
        if ($this->attr['savedStateLoad'] == 'true')
            return true;
        else
            return false;
    }

    /**
     * set column widths
     * the set value should be a string ex "255,255,255" if columns no = 3
     * the set value should be a string ex "255,255,255,350" if columns no = 4
     * @access public
     * @param string $column_widths
     * @return none
     */
    public function set_col_widths($col_widths) {
        $this->attr['colWidths'] = json_encode($col_widths);
        return $this;
    }

    /**
     *
     * @return string colWidths
     */
    public function get_col_widths() {
        return $this->attr['colWidths'];
    }

    /**
     * set header alignments
     * @access public
     * @param string $colAligns
     * @return none
     */
    public function set_header_aligns($header_aligns) {
        $this->attr['headerAligns'] = json_encode($header_aligns);
        return $this;
    }

    /**
     *
     * @return string headerAligns
     */
    public function get_header_aligns() {
        return $this->attr['headerAligns'];
    }

    /**
     * set col min width
     * @access public
     * @param string $colmin_widths
     * @return none
     */
    public function set_col_min_widths($col_min_widths) {
        $this->attr['colMinWidths'] = json_encode($col_min_widths);
        return $this;
    }

    /**
     *
     * @return string col_min_widths
     */
    public function get_col_min_widths() {
        return $this->attr['colMinWidths'];
    }

    /**
     * set col min width
     * @access public
     * @param string $colis_resizables
     * @return none
     */
    public function set_col_is_resizables($col_is_resizables) {
        $this->attr['colIsResizables'] = json_encode($col_is_resizables);
        return $this;
    }

    /**
     *
     * @return string col_is_resizables
     */
    public function get_col_is_resizables() {
        return $this->attr['colIsResizables'];
    }

    /**
     * set column alignments
     * @access public
     * @param string $colAligns
     * @return none
     */
    public function set_column_aligns($column_aligns) {
        $this->attr['colAligns'] = json_encode($column_aligns);
        return $this;
    }

    /**
     *
     * @return string colAligns
     */
    public function get_column_aligns() {
        return $this->attr['colAligns'];
    }

    /**
     * set columns params which should be equals to the field_name in the database .
     * these params are used to order the grid according to a certain field
     *  the set value should be a string ex "'tya','Ny','Ncaa'"
     * where tya is the name of the first column in the database
     * and Ny is the name of the second column in the database
     * and so on
     *
     * @access public
     * @param int $min_col_width
     * @return none
     */
    public function set_col_sort_params($col_sort_params) {
        $this->attr['colSortParams'] = json_encode($col_sort_params);
        return $this;
    }

    /**
     *
     * @return string colSortParams
     */
    public function get_col_sort_params() {
        return $this->attr['colSortParams'];
    }

    /**
     * set columns min width
     * @access public
     * @param int $min_col_width
     * @return none
     */
    public function set_min_col_width($min_col_width) {
        $this->attr['minColWidth'] = $min_col_width;
        return $this;
    }

    /**
     *
     * @return minColWidth
     */
    public function get_min_col_width() {

        return $this->attr['minColWidth'];
    }

    /**
     * set columns min width
     * @access public
     * @param int $default_params
     * @return none
     */
    public function set_default_params($default_params) {
        $this->attr['defaultParams'] = $default_params;
        return $this;
    }

    /**
     *
     * @return minColWidth
     */
    public function get_default_params() {

        return $this->attr['defaultParams'];
    }

    /**
     * params will be sent throught request and sent to server
     * @access public
     * @param string $param
     * @return none
     */
    public function add_params($params = null) {
        array_push($this->params, $params);
    }

    /**
     *
     * @return string params
     */
    private function get_params() {
        return "{" . implode(",", $this->params) . "}";
    }

    /**
     * extra params will be sent throught request and sent to server
     * @access public
     * @param string $extra_param
     * @example add_extra_params($extra_param="comp_name:'opentechcom'")
     * @return none
     */
    public function add_extra_params($extra_params = null) {
        array_push($this->extra_params, $extra_params);
        return $this;
    }

    /**
     *
     * @return string extraParams
     */
    private function get_extra_params() {
        if ($this->extra_params)
            return "extraParams:{" . implode(',', $this->extra_params) . "},";
        return null;
    }

    /**
     * this function is used to set_header the header columns
     * it uses an array to load the needed data
     * this column will be loaded later on when the grid loads on page loading
     * @access public
     * @param array $header
     * @example $search_parser->set_header( array(  'db_name'=>$db_name,
      'title'=>$title,
      'width'=>$width);
     *                                      );
     */
    public function set_header($header = array()) {
        $col_widths = array();
        $col_min_widths = array();
        $col_is_resizables = array();
        $arr_alias = array();
        $column_aligns = array();
        $header_aligns = array();
        foreach (@$header as $key => $array) {

            $this->add_header_titles($array['title']);
            $this->add_header_columns($array['db_name']);

            array_push($col_widths, $array['width']);
            if (!$array['min_width'])
                $array['min_width'] = $this->get_min_col_width();
            array_push($col_min_widths, $array['min_width']);
            if ($array['is_resizable'] == "")
                $array['is_resizable'] = $this->get_is_resizable_cols();
            array_push($col_is_resizables, $array['is_resizable']);

            array_push($header_aligns, $array['header_align']);
            array_push($column_aligns, $array['column_align']);
            if ($array['type'])
                $this->add_params("header_" . $array['db_name'] . "_type:" . json_encode($array['type']));
            if ($array['alias'])
                $this->add_params("header_" . $array['db_name'] . "_alias:" . json_encode($array['alias']));
            if ($array['alias'])
                array_push($arr_alias, $array['alias'] . ' as ' . $array['db_name']);
            else
                array_push($arr_alias, $array['db_name']);
        }
        $this->set_col_widths($col_widths);
        $this->set_col_min_widths($col_min_widths);
        $this->set_col_is_resizables($col_is_resizables);
        $this->set_column_aligns($column_aligns);
        $this->set_header_aligns($header_aligns);
        $this->add_params("select_fields:" . json_encode($arr_alias) . "");
        $this->set_col_sort_params($this->get_header_columns());
        return $this;
    }

    /**
     * @access private
     * @param string $value
     */
    private function add_header_columns($value) {
        array_push($this->header_columns, $value);
        return $this;
    }

    /**
     * get the databasee table name which is the source of the grid results
     * @access private
     * @param none
     * @return header_columns as a array
     */
    public function get_header_columns() {
        return $this->header_columns;
    }

    /**
     * @access private
     * @param string $value
     */
    private function add_header_titles($value) {
        array_push($this->header_titles, $value);
        return $this;
    }

    /**
     * @access public
     * @param none
     * @return header_titles as a array
     */
    public function get_header_titles() {
        return $this->header_titles;
    }

    /**
     * defines the databasee table name which is the source of the grid results
     * @access public
     * @param string $db_table_name
     * @return none
     */
    public function set_db_table_name($db_table_name) {
        $this->db_table_name = $db_table_name;
        return $this;
    }

    /**
     * get the databasee table name which is the source of the grid results
     * @access public
     * @param none
     * @return $db_table_name databasee table name
     */
    public function get_db_table_name() {
        return $this->db_table_name;
    }

    /**
     * get seach js attributes
     * @return string $attr
     */
    public function get_attr() {

        $attr_str = null;

        foreach (@$this->attr as $key => $attr) {
            $attr_str .= "$key:$attr,";
        }
        $attr_str .= $this->get_extra_params();
        $attr_str = substr($attr_str, 0, -1);
        return $attr_str;
    }

    /**
     * this function is used to set_filter the filter columns
     * it uses an array to load the needed data
     * this column will be loaded later on when the grid loads on page loading
     * @access public
     * @param array $filter
     * @example $search_parser->set_filter( array(  'form'=>$form,
      'parms'=>$parms,);
     *                                      );
     */
    public function set_filter($filter = array()) {
        $this->filte_form = $filter['form'];
        foreach (@$filter['params'] as $key => $value)
            $this->add_params($value);
        return $this;
    }

    /**
     *
     * @return string $filte_form
     */
    private function get_filter_form() {
        return $this->filte_form;
    }

    /**
     * defines the location of modify form , in order to direct the record to its link
     * @access public
     * @param string $modify_location
     * @return none
     */
    public function set_modify_iframe($modify_iframe = null) {
        $this->modify_iframe = $modify_iframe;
        $this->set_editing_target();
        return $this;
    }

    /**
     * returns the location of modify form , in order to direct the record to its link
     * @access public
     * @return string $modify_location
     */
    public function get_modify_iframe() {
        return $this->modify_iframe;
    }

    /**
     * load grid function , it loads the grid by calling all the user defined grid features and settings
     * @access public
     * @param none
     */
    public function load() {
        $this->add_params("header:" . json_encode($this->get_header_columns()));
        foreach (@$this->get_header_titles() as $value) {
            $th .= '<th>' . $value . '</th>';
        }

        $button_clear = new button($this->get_name() . '-filter-clear-bt');
        $button_clear->set_text(LANG_CLEAR);
        $button_clear->add_event("onclick", "clearForm(this.form)");

        $button_hide = new button($this->get_name() . '-filter-hide-bt');
        $button_hide->set_text(LANG_HIDE);

        $button_search = new button($this->get_name() . '-filter-search-bt');
        $button_search->set_text(LANG_SEARCH);

        $this->add_params('records_per_page:' . $this->get_records_per_page());
        $this->set_default_params($this->get_params());

        $show_iframe = null;
        if (!$this->get_enable_debug()){
            $show_iframe = 'none';
        }
        ob_start();
        ?>
        <div>
            <div id="<?= $this->get_name() ?>_filter_div">
                <div id="<?= $this->get_name() ?>_filter_form_div_container">
                    <form method="GET" name="search_form" class="" onsubmit="return false;">
                        <div id="<?= $this->get_name() ?>_filter_form_div">
                            <div style="padding-bottom:3px;"></div>
        <?= $this->get_filter_form() ?>
                        </div>
                        <div id="<?= $this->get_name() ?>_filter_button_div"><?= $button_search->load() ?>&nbsp;&nbsp;<?= $button_clear->load() ?>&nbsp;&nbsp;<?= $button_hide->load() ?></div>
                    </form>
                </div>
            </div>
            <table id="<?= $this->get_name() ?>">
                <thead>
                    <tr>
        <?= $th ?>
                    </tr>
                </thead>
                <tbody>

                </tbody>
            </table>
            <div id="<?= $this->get_name() ?>-export-pdf-xls-div" style="display:<?= $show_iframe ?>;width:1px;height:1px;">
                <iframe id="<?= $this->get_name() ?>-export-xlspdf-iframe" width="1200" name="<?= $this->get_name() ?>-export-xlspdf-iframe" style="display:<?= $show_iframe ?>"></iframe>
            </div>
        </div>
        <script type="text/javascript">
        var <?= $this->get_name() ?>grid;
        $(document).ready(
                function() {
                        <?= $this->get_name() ?>grid = $("#<?= $this->get_name() ?>").ingrid({<?= $this->get_attr() ?>});
                        $("button[name=<?= $this->get_name() ?>-filter-hide-bt]").bind("click", function(event){
                            <?= $this->get_name() ?>grid.pb5.click();
                            $(this).parents("form:first").find("input:text").first().focus();
                        });
                        $("#<?= $this->get_name() ?>_filter_form_div_container input:text").bind("keydown", function(e){
                            if (e.keyCode == 13)
                                $("#<?= $this->get_name() ?>-filter-search-bt").click();
                        });
                        <?php if($this->get_request_on_load()):?>
                            <?= $this->get_name() ?>grid.g.load(<?= $this->get_params() ?>);
                        <?php endif;?>
                        $("a[rel='FormTabContainer-tabcontainer1']").click(function(){

                            <?= $this->get_name() ?>grid.g.load(<?= $this->get_params() ?>);
                            $(".grid-records-count-info").show();
                        });
                        $("#FormTabContainer-tab-container a").not("a[rel='FormTabContainer-tabcontainer1']").click(function(){
                            $(".grid-records-count-info").hide();
                        });
                        $("#<?= $this->get_name() ?>-quicksearch-reload").click(function(){
                            $("#<?= $this->get_name() ?>-quicksearch").val("");
                            clearForm(document.search_form);
                            <?php $this->add_params('page:1') ?>
                            <?= $this->get_name() ?>grid.g.p.find(".grid-page-prev").addClass("grid-page-nav-disabled");
                            $("#<?= $this->get_name() ?>-filter-search-bt").click();
                        });
                        $("#<?= $this->get_name() ?>-quicksearch").bind("keydown", function(e){
                            <?php $this->add_params('quicksearch:$("#' . $this->get_name() . '-quicksearch").val()') ?>
                            clearForm(document.search_form);
                            if (e.keyCode == 13)
                                $("#<?= $this->get_name() ?>-filter-search-bt").click();
                        });
                        $("#<?= $this->get_name() ?>-filter-search-bt").click(function(event){
                            <?= $this->get_name() ?>grid.pb5.searchAction();
                            <?= $this->get_name() ?>grid.g.p.find("input.grid-page-input").val(1);
                            <?= $this->get_name() ?>grid.g.load(<?= $this->get_params() ?>);
                        });
                        <?php $this->add_params('header_titles:[' . json_encode($this->get_header_titles()) . ']');
                        if ($this->get_show_export_Xls()):
                         ?>
                        $("#<?= $this->get_name() ?>exportXlsButton").click(function(){
                            <?php
                            $this->add_params('export_file:"xls"');
                            $this->add_params('action:"export"');
                            ?>
                            <?= $this->get_name() ?>grid.g.export_file(<?= $this->get_params() ?>);
                        });
                        <?php endif; ?>
                    });
        </script>
        <?php
    $html = ob_get_contents();
    ob_clean();
    
    
    return $html;
    }

}
?>