<?php

/**
 * file_upload class . use this class to create file_upload object
 * @author Ihab A. Ali
 * @copyright copyright (c) copyright 2009 
 * @license GNU Public Licence (GPL)
 * @example $this->menu_icon = new file_upload("menu_icon");
  $this->menu_icon->set_db_table_name("menu");
  $this->menu_icon->set_extensions(icon_ext);
  $this->menu_icon->set_size(100);
  $this->menu_icon->set_valid(true);
  $this->menu_icon->set_title(menu_icon.' '.LANG_VALID);
  to load in add or modify form use :
  $this->menu_icon->set_value($rows['menu_icon']);
  $this->menu_icon->load_file();
  to add a file (upload it to server) use :
  $this->menu_icon->set_record_id($this->form_parser->get_last_id());
  $this->menu_icon->add_file();
  to modify file use :
  $this->menu_icon->modify_file();
  to remove file use :
  $this->menu_icon->remove_file();
  $this->form_parser->delete("menu");
 *
 */
class input_file extends builder_elements {

    private $width = 280;

    /**
     *
     * @var object sql
     */
    private $sql = null;

    /**
     * allowed extensions
     * @access private
     * @var string $extensions
     */
    private $extensions = null;

    /**
     * the allowed size of the file
     * @access private
     * @var string $file_size
     */
    private $file_size = 100;

    /**
     * @access private
     * @var int $record_id
     */
    private $record_id = null;

    /**
     * data base table name
     * @access private
     * @var string $db_table_name
     */
    private $db_table_name = null;

    /**
     * data file direction
     * @access private
     * @var string $file_direction
     */
    private $file_directory = null;

    /**
     * @access public
     * @var bool debug
     */
    private $debug = false;

    /**
     * to set the returned message
     * @access public
     * @var string return
     */
    private $return = null;

    /**
     * Read Only
     * @access public
     * @var type 
     */
    private $readonly = false;

    /**
     * construct function
     * @access public
     * @param string $name to set the name
     */
    public function __construct($name = null) {
        parent::set_name($name);
        $this->sql = new sql();
    }

    /**
     * @access private
     * @param string $extensions
     */
    public function set_extensions($extensions) {
        $this->extensions = $extensions;
        return $this;
    }

    /**
     * @access private
     * @return string $extensions
     */
    public function get_extensions() {
        return $this->extensions;
    }

    /**
     * @access private
     * @param string $size
     */
    public function set_size($file_size = 100) {
        $this->file_size = $file_size;
        return $this;
    }

    /**
     * @access private
     * @return string $size
     */
    public function get_size() {
        return $this->file_size;
    }

    /**
     * @access private
     * @return string $valid
     */
    public function get_valid() {
        if (parent::get_valid() == true)
            return "valid-file-" . $this->get_extensions();
        else
            return "format-file-" . $this->get_extensions();
    }

    public function set_width($width = 155) {
        $this->width = $width;
        return $this;
    }

    /**
     * @access private
     * @return int $width
     */
    public function get_width() {
        return $this->width;
    }

    /**
     * set the last insert id
     * @param int $record_id
     */
    public function set_record_id($record_id) {
        $this->record_id = $record_id;
        return $this;
    }

    /**
     * get the last insert id
     * @return int $record_id
     */
    public function get_record_id() {
        return $this->record_id;
    }

    /**
     * set database table name
     * @param string $db_table_name
     */
    public function set_db_table_name($db_table_name) {
        $this->db_table_name = $db_table_name;
        return $this;
    }

    /**
     * get db_table_name
     * @return string $db_table_name
     */
    public function get_db_table_name() {
        return $this->db_table_name;
    }

    /**
     * set file directory on server
     * @param string $file_directory
     */
    public function set_file_directory($file_directory) {
        $this->file_directory = $file_directory;
        return $this;
    }

    /**
     * get file directory on server
     * @return string $file_directory
     */
    public function get_file_directory() {
        return $this->file_directory;
    }

    /**
     * set returned message
     * @param string $return
     */
    public function set_return($return) {
        $this->return = $return;
        return $this;
    }

    /**
     * get return message
     * @return string $return
     */
    public function get_return() {
        return $this->return;
    }

    /**
     * set readonly
     * @param string $readonly
     */
    public function set_readonly($readonly = true) {
        $this->readonly = $readonly;
        return $this;
    }

    /**
     * get readonly message
     * @readonly string $readonly
     */
    public function get_readonly() {
        return $this->readonly;
    }

    /**
     *
     * @return string $html_code to load the upload_object if it was on add form
     */
    public function load() {
        $span = null;
        if (parent::get_valid()) {
            $span = $this->get_required_span();
        }
//        if (parent::get_disable()) {
//            anti_inspection::push_disabled_elments(parent::get_name(), parent::get_value(),"input_file");
//        }
        $file_extension = explode(".", parent::get_value());
        $view_button = null;
        if (strtolower($file_extension[1]) == "jpg" || strtolower($file_extension[1]) == "png" || strtolower($file_extension[1]) == "jpeg" || strtolower($file_extension[1]) == "gif") {
            $view_button = '<button  type="button" onclick="show_file(' . parent::get_name() . '_img)">' . LANG_VIEW . '</button>';
        }
        $delete_button = '<button type="button" id="' . parent::get_id() . '_to_delete_button">' . LANG_DELETE . '</button>';

        if ($this->get_readonly())
            $delete_button = '';

        ob_start();
        ?>

        <div style="position:relative;display:inline-flex;width:fit-content;" class="input_file">
            <div id="<?= parent::get_name() ?>_div_0" style="display: <?= parent::get_value() ? "none" : "flex" ?>">
                <div style="display:flex;width:fit-content" id="<?= parent::get_id() ?>_mask_container" class="input_file_mask_div">
                    <input type="text" name="mask_<?= parent::get_name() ?>" id="mask_<?= parent::get_id() ?>" style="width:calc(<?= $this->get_width() ?>px - 13px);padding-<?= DIR_RALIGN ?>:22px;" class="<?= parent::get_value() ? "" : $this->get_valid() ?>"  placeholder="<?= LANG_SIZE . ':' . $this->get_size() . ' ' . LANG_EXT . ':' . $this->get_extensions() ?>"/>
                </div>
                <div class="file_container file_container_<?= dir ?>  mdi mdi-upload">
                </div>
                <div class="file_sub_container" id="<?= parent::get_id() ?>_file_sub_container" style="width: 100%">
                    <input type="file" class="input_upload" name="<?= parent::get_name() ?>" id="<?= parent::get_id() ?>" width="30" />
                </div>
            </div>
            <div id="<?= parent::get_name() ?>_div_1" style="position:relative;width: fit-content;display: <?= parent::get_value() ? "flex" : "none" ?>">
                <div style="display:flex;width: fit-content;">
                    <button type="button"  onclick="window.open('<?= upload_url_path . parent::get_value() ?>', '')"><?= LANG_OPEN ?></button>
                    <?= $view_button ? '&nbsp;' . $view_button : '';
                    echo '&nbsp;' . $delete_button ?>    
                </div>
                <input type="hidden" name="<?= parent::get_name() ?>_to_delete" id="<?= parent::get_id() ?>_to_delete" value="">
                <input type="hidden" name="<?= parent::get_name() ?>_actual_value" id="<?= parent::get_id() ?>_actual_value" value="<?= parent::get_value() ?>">
                <input type="hidden" name="<?= parent::get_name() ?>_modify_value" id="<?= parent::get_id() ?>_modify_value" value="<?= parent::get_value() ?>">
            </div>

        </div><?= $span ?>
        <script type="text/javascript">
        <?php if (parent::get_value()): ?>
                var <?= parent::get_name() ?>_img = new Image();
            <?= parent::get_name() ?>_img.src = "<?= upload_url_path . parent::get_value() ?>";
        <?php endif; ?>
                        $(document).ready(function () {
                            $("#<?= parent::get_id() ?>_required_alert").attr("id", "mask_<?= parent::get_name() ?>_required_alert");
        <?php if (parent::get_value()): ?>
                                $("#mask_<?= parent::get_id() ?>_required_alert").hide();
        <?php endif; ?>
                            var file_selected=function() {
                                $("#<?= parent::get_id() ?>").click(function () {
        <?= $this->attr['onclick'] ?>
                                }).bind("change", function () {
                                    var obj = $(this);
                                    $(obj).parent().parent().find(".file_container").removeClass("mdi-upload").addClass("mdi-close-circle").css("z-index","15").click(function () {
                                        $(this).removeClass("mdi-close-circle").addClass("mdi-upload").css("z-index","");
                                        $("#<?= parent::get_id() ?>").remove();
                                        var $input = $("<input type='file' name='<?= parent::get_name() ?>'  id='<?= parent::get_id() ?>' />");
                                        $input.addClass("input_upload");
                                        $("#<?= parent::get_id() ?>_file_sub_container").prepend($input);
                                        $("#mask_<?= parent::get_name() ?>").val("");
                                        file_selected();
                                        $(this).unbind("click");
                                    });
                                    $("#mask_<?= parent::get_name() ?>").val($(obj).val());
                                    $("#mask_<?= parent::get_name() ?>_required_alert").css("color", "");
                                    $("#mask_<?= parent::get_name() ?>_required_alert").attr('data-original-title', 'Required');
                                    $("#mask_<?= parent::get_name() ?>").removeClass("required");
                                    $("#<?= parent::get_id() ?>").hide();
        <?= $this->attr['onchange'] ?>
                                });
                            }
                            file_selected();
                            $("#<?= parent::get_name() ?>_to_delete_button").bind("click", function () {
                                load_confirm("<?= LANG_IMAGE_DELETE_MSG ?>", function () {
                                    $("#<?= parent::get_name() ?>_div_1").hide();
                                    $("#<?= parent::get_name() ?>_to_delete").val("<?= $this->get_value() ?>");
                                    $("#<?= parent::get_name() ?>_modify_value").val("");
                                    $("#<?= parent::get_name() ?>_div_0").show();
                                    $("#mask_<?= parent::get_name() ?>").addClass("<?= $this->get_valid() ?>");
                                    $("#mask_<?= parent::get_id() ?>_required_alert").css("color", "");
                                    $("#mask_<?= parent::get_id() ?>_required_alert").show();
                                    mabvalid.init(document.getElementById("<?= parent::get_id() ?>_mask_container"));
                                }, function () {});
                            });
                        });
        </script>
        <?php
        $html = ob_get_contents();
        ob_clean();
        return $html;
    }

    /**
     * @access public
     * function add_file , uploads file to the server
     */
    public function add() {

        if ($GLOBALS['_FILES'][parent::get_name()]['name']) {
            $extension = explode("|", strtolower($this->get_extensions()));
            $type = explode(".", $GLOBALS['_FILES'][parent::get_name()]['name']);
            $type = strtolower($type[count($type) - 1]);
            $size = round($GLOBALS['_FILES'][parent::get_name()]['size'] / 1024);

            $file_dir = $this->get_db_table_name();
            if ($this->get_file_directory()) {
                $file_dir = $this->get_file_directory();
            }
            $current_month = date("Y-m");
            $file_rename = $file_dir . "/" . $current_month . "/" . parent::get_name() . "_" . $this->get_record_id();
            $i = 0;
            while (file_exists(upload_physical_path . $file_rename . "." . $type)) {
                ++$i;
                $file_rename .= "_" . $i;
            }
            $file_rename .= "." . $type;
            $current_file = upload_physical_path . $file_rename;

            $new_file_dir = upload_physical_path . $file_dir;
            $current_month = $new_file_dir . "/" . $current_month;

            if (!is_numeric($size)) {
                $this->set_return("SIZE IS NOT NUMERIC");
                return false;
            }
            if ($size > $this->get_size()) {
                $this->set_return(MSG_UPLOAD_LARGE_FILE);
                return false;
            }

            if (!in_array(strtolower($type), $extension)) {
                $this->set_return(MSG_UPLOAD_UNVALID_FILE);
                return false;
            }
            if (!is_dir($new_file_dir)) {
                mkdir($new_file_dir, 0777);
            }
            if (!is_dir($current_month)) {
                mkdir($current_month, 0777);
            }
            move_uploaded_file($GLOBALS['_FILES'][parent::get_name()]['tmp_name'], $current_file);
            chmod($current_file, 0777);
            $this->sql->execute_statment("update `" . $this->get_db_table_name() . "` set `" . parent::get_name() . "` = '$file_rename'
                                          where `" . $this->get_db_table_name() . _id . "` = " . $this->get_record_id(), $this->get_debug());
        }

        return true;
    }

    /**
     * modify_file
     * @access public
     * @return bool $modified
     */
    public function modify() {
        if (!$this->get_record_id()) {
            $this->set_record_id($GLOBALS['_POST'][$this->get_db_table_name() . _id]);
        }
        if ($GLOBALS['_POST'][parent::get_name() . '_to_delete']) {
            @unlink(strtolower(upload_physical_path . $GLOBALS['_POST'][parent::get_name() . '_to_delete']));
            $this->sql->execute_statment("update " . $this->get_db_table_name() . " set " . parent::get_name() . " = ''
                                    where " . $this->get_db_table_name() . _id . " = " . $this->get_record_id(), $this->get_debug());
        }
        if ($GLOBALS['_FILES'][parent::get_name()]['name']) {
            return $this->add();
        }
        if ($_POST[parent::get_name() . '_modify_value']) {
            return $this->sql->execute_statment("update `" . $this->get_db_table_name() . "` set `" . parent::get_name() . "` = '" . $_POST[parent::get_name() . '_modify_value'] . "'
                                    where `" . $this->get_db_table_name() . _id . "` = '" . $this->get_record_id() . "'", $this->get_debug());
        }
    }

    /**
     * remove file from server
     * @access public
     */
    public function delete() {
        if (!$this->get_record_id()) {
            $this->set_record_id($GLOBALS['_POST'][$this->get_db_table_name() . _id]);
        }
        if (!$GLOBALS["_POST"][parent::get_name() . '_actual_value']) {
            $reuslt = $this->sql->get_result("select * from " . $this->get_db_table_name() . " where " . $this->get_db_table_name() . _id . "
                                    = " . $this->get_record_id(), $this->get_debug());
            $rows = $this->sql->next($reuslt);
            $file_to_delete = $rows[parent::get_name()];
        } else {
            $file_to_delete = $GLOBALS["_POST"][parent::get_name() . '_actual_value'];
        }
        @unlink(strtolower(upload_physical_path . $file_to_delete));
    }

    /**
     * @access public
     * @param string $debug
     * @return none
     */
    public function debug($debug = true) {
        $this->debug = $debug;
        return $this;
    }

    /**
     * @access public
     * @return get_debug
     */
    public function get_debug() {
        return $this->debug;
    }

}
?>