<?php

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 * Description of parser_input_radio
 *
 * @author ihab
 */
class builder_radio extends builder_input {

    /**
     * class __construct
     */
    public function __construct($name) {
        parent::set_name($name);
        parent::set_type("radio");
        parent::set_width(null);
    }

    public function get_valid() {
        parent::add_class('radio');
    }

}

class input_radio {

    /**
     * used to set the name of radios
     * @var string name
     */
    private $name = null;

    /**
     * will be used to store radio buttons
     * @var array $radio
     */
    private $option = array();

    /**
     * to set actual value
     * @var string $value
     */
    private $value = null;

    /**
     *
     * @param horizantal 'h' vertical 'v'
     */
    private $vertical = false;

    /**
     * set the default checked value
     * @access private
     * @var string $default
     */
    private $default = null;

    /**
     *
     * @var bool $disable
     */
    private $disable = false;

    /**
     *
     * @var bool $readonly
     */
    private $readonly = false;

    /**
     * __construct
     * @param string $name
     */
    public function __construct($name) {
        $this->set_name($name);
    }

    /**
     *
     * @param string $name
     */
    public function set_name($name) {
        $this->name = $name;
        return $this;
    }

    /**
     *
     * @return string $name
     */
    public function get_name() {
        return $this->name;
    }

    /**
     * function set_value overrides parent::set_value
     * to set the real value after modify and get if checked or not
     * @access public
     * @param string $value
     */
    public function set_value($value = null) {
        $this->value = $value;
        return $this;
    }

    /**
     *
     * @return string $value
     */
    public function get_value() {
        return $this->value;
    }

    /**
     * set default checked value
     * @access public
     * @param string $default
     */
    public function set_default($default) {
        $this->default = $default;
        return $this;
    }

    /**
     *
     * @return string $default
     */
    public function get_default() {
        return $this->default;
    }

    /**
     * @access public
     */
    public function add_option($label, $value, $script = null) {
        $arr = array('label' => $label, 'value' => $value, 'script' => $script);
        array_push($this->option, $arr);
        return $this;
    }

    /**
     * is vertical align?
     * @param bool $vertical
     */
    public function set_vertical($vertical = true) {
        $this->vertical = $vertical;
        return $this;
    }

    /**
     *
     */
    private function get_vertical() {
        if (!$this->vertical)
            return "float:" . DIR_ALIGN;
    }

    /**
     * @access public
     * @param string $disable
     */
    public function set_disable($disable = array()) {
        if(is_array($disable)){
            $this->disable = $disable;
            return $this;        
        }
        
        //anti injection need
        exit("Radio set disable should array not bool");
    }

    /**
     * @access public
     * @return string $disable
     */
    public function get_disable() {
        if ($this->disable)
            return $this->disable;

        return false;
    }

    /**
     * @access public
     * @param string $readonly
     */
    public function set_readonly($readonly = true) {
        $this->readonly = $readonly;
        return $this;
    }

    /**
     * @access public
     * @return string $readonly
     */
    public function get_readonly() {
        if ($this->readonly)
            return $this->readonly;
        return false;
    }

    /**
     * @access public
     * @param string $width
     */
    public function set_width($width = true) {
        $this->width = $width;
        return $this;
    }

    /**
     * @access public
     * @return string $width
     */
    public function get_width() {
        return $this->width;
    }

    /**
     *
     */
    public function load() {
        $html = "<ul style='margin:0px;padding:0px;list-style:none;white-space:nowrap;width:{$this->get_width()}px'>";
        $disable = $this->get_disable();
        $readonly = $this->get_readonly();
        foreach ($this->option as $key => $option) {
            $radio = new builder_radio($this->get_name());
            $radio->set_id($this->get_name() . '_' . $option['value']);
            $radio->set_value($option['value']);
            if (!$this->get_value() AND $this->get_default() AND $this->get_default() == $option['value'])
                $radio->add_attr("checked", "checked");
            if (($this->get_value() OR $this->get_value() == 0) AND $this->get_value() == $option['value'])
                $radio->add_attr("checked", "checked");
            if ($option['script'])
                $radio->add_event('onclick', $option['script']);
            if ($disable[$key] == true OR ( is_bool($disable) AND $disable == true))
                $radio->set_disable();
            if ($readonly[$key] == true OR ( is_bool($readonly) AND $readonly == true)) {
                $radio->set_disable();
//                if($this->get_value()){
//                    $html.="<input type='hidden' name='{$this->get_name()}' id='{$this->get_name()}' value='{$this->get_value()}'/>";
//                }
            }

            $html .= "<li style='margin:0px;padding:0px;" . $this->get_vertical() . "'>
                        <table cellpadding='0' cellspacing='0' style='display:inline;' valign='top'>
                            <tr>
                                <td valign='middle'  style='white-space:nowrap;'><label>" . $option['label'] . "</label></td>
                                <td>&nbsp;</td>
                                <td  valign='middle' style='white-space:nowrap;'>" . $radio->load() . "</td>
                                <td>&nbsp;&nbsp;</td>
                            </tr>
                        </table>
                     </li>";
        }
        $html .= "</ul>
                <div style='clear:both;'></div>";
        if ($readonly) {
            anti_inspection::push_disabled_elments($this->get_name(), $this->get_value(), "radio");
        }
        return $html;
    }

}

?>