<?php

/**
 * last update Sat Jul 10,2010
 * select select class . use this class build combo class
 * inherited from sql_class 
 * @license GNU Public Licence (GPL)
 * @example  $html = new select($name,$option_view=null,$option_value=null)
 * @example
  $new_list=new multilist("objectname","optionaname","optionvalue");
  $new_listset->set_size(15);
  $new_listset->height(200);
  $new_listset->width(200);
  $new_list->set_filter();
  $new_list->set_value($rows['category_level2_id']);
  $new_list->set_data_source("select * from category_level2");
 */
class multilist extends data_source {

    /**
     *
     * @var int width
     */
    private $width = 180;

    /**
     * @access private
     * @var int height
     */
    private $height = 200;

    /**
     *  allow list to list filter
     * @var bool multi_filter
     */
    private $multi_filter = true;
    private $list_value = array();

    /**
     * class construct
     * @param string $name
     * @param string $option_view
     * @param string $option_value
     */
    public function __construct($name, $option_view = null, $option_value = null) {
        parent::__construct($option_view, $option_value);
        parent::set_name($name);
        parent::add_attr("size", "15");
    }

    /**
     * set object height
     * @access public
     * @param int $height
     */
    public function set_height($height = 200) {
        $this->height = $height;
        return $this;
    }

    /**
     * @access public
     * @return int $height returns select height
     */
    public function get_height() {
        return $this->height;
    }

    /**
     *
     * set if should filter the mutli select
     * @access public
     * @param bool $multi_filter
     */
    public function set_filter($multi_filter = true) {
        $this->multi_filter = $multi_filter;
        return $this;
    }

    /**
     *
     * @access public
     * @return bool $this->multi_filter
     */
    public function get_filter() {
        return $this->multi_filter;
    }

    /**
     *
     * @param int $size
     */
    public function set_size($size) {
        parent::add_attr("size", $size);
        return $this;
    }

    /**
     * @access public
     * @return bool $valid returns select validation
     */
    public function get_valid() {
        if (parent::get_valid()) {
            return 'valid';
        }
        return false;
    }

    /**
     * @access public
     * @return bool $disable returns select disable
     */
    public function get_disable() {
        if (parent::get_disable()) {
            return 'disabled="disabled"';
        }
        return false;
    }

    /**
     * set the width of the combo
     * @access public
     * @param string $width
     */
    public function set_width($width = null) {
        $this->width = $width;
        return $this;
    }

    /**
     * @access public
     * @return string $width
     */
    public function get_width() {
        return $this->width;
    }

    /**
     * @access private
     * @return array $options
     */
    public function get_options() {
        $newvalue = parent::get_value();

        if ($newvalue && !is_array($newvalue)) {
            $newvalue = explode(",", $newvalue);
        }
        foreach (parent::get_data_set() as $value => $view) {
            if (in_array($value, $newvalue)) {
                $this->list_value[] = $value;
                $options[2] .= '<option value="' . $value . '">' . $view . '</option>';
            } else {
                $options[1] .= '<option value="' . $value . '">' . $view . '</option>';
            }
        }
        parent::set_value(implode(',', $this->list_value));
        return $options;
    }

    /**
     *
     * @param <type> $data_source
     * @return <type>
     */
    public function load() {

        $data_source = $this->get_options();
        if ($this->get_filter()) {
            $filter_width = $this->get_width() - 20;
            $filter = '<input type="text" id="' . parent::get_id() . 'filterinput" name="' . parent::get_id() . 'filterinput"  style="width:' . $filter_width . 'px;" placeholder="' . LANG_SEARCH . '">';
        } else {
            $filter = '';
        }
        if (is_array($this->get_value())) {
            $this->set_value(implode(',', $this->get_value()));
        }
        ob_start();
        ?>
        <div id="<?= parent::get_id() ?>_container" class="multilist">
            <table border="0" cellpadding="0" cellspacing="0" dir="<?= dir ?>">
                <tr>
                    <td align=center><label><?= Constant(parent::get_id()) ?> <?= MULTI_LISTED ?></label></td>
                    <td></td>
                    <td align=center><label><?= Constant(parent::get_id()) ?> <?= MULTI_SELECTED ?></label></td>
                    <?php if ($this->get_valid()): ?>
                        <td rowspan="4" valign="middle">
                            <i style="padding:2px;"></i><?= $this->get_required_span() ?>
                        </td>
                    <?php endif; ?>
                </tr>
                <tr>
                    <td colspan="3">
                        <div class='multilist_filter_div'>
                            <?= $filter ?>
                            <i class="mdi mdi-filter"></i>
                        </div>

                    </td>
                </tr>
                <tr>
                    <td valign="bottom">
                        <select  id="<?= parent::get_id() ?>multilist_listed" <?= $this->get_disable() ?> name="<?= parent::get_id() ?>multilist_listed"  style="width:<?= $this->get_width() ?>px;height:<?= $this->get_height() ?>px" multiple>
                            <?= $data_source[1] ?>
                        </select>
                    </td>
                    <td valign=middle>
                        <table border="0" cellspacing="10" cellpadding="0">
                            <tr><td><button class="<?= parent::get_id() ?>addbutton"    <?= $this->get_disable() ?> type="button"  style="width:40px;" id="<?= parent::get_id() ?>add"      ><i class="mdi mdi-arrow-<?=DIR_RALIGN?>"></i></button></td></tr>
                            <tr><td><button class="<?= parent::get_id() ?>addbutton"    <?= $this->get_disable() ?> type="button"  style="width:40px;" id="<?= parent::get_id() ?>addall"   ><i class="mdi mdi-arrow-collapse-<?=DIR_RALIGN?>"></i></button></td></tr>
                            <tr><td><button class="<?= parent::get_id() ?>deletebutton" <?= $this->get_disable() ?> type="button"  style="width:40px;" id="<?= parent::get_id() ?>delete"   ><i class="mdi mdi-arrow-<?=DIR_ALIGN?>"></i></button></td></tr>
                            <tr><td><button class="<?= parent::get_id() ?>deletebutton" <?= $this->get_disable() ?> type="button"  style="width:40px;" id="<?= parent::get_id() ?>deleteall"><i class="mdi mdi-arrow-collapse-<?=DIR_ALIGN?>"></i></button></td></tr>
                        </table>
                    </td>
                    <td valign="bottom"  style="height:<?= $this->get_height() ?>px">
                        <select <?= $this->get_disable() ?> id="<?= parent::get_id() ?>multilist_selected" name="<?= parent::get_id() ?>multilist_selected"  style="width:<?= $this->get_width() ?>px;height:<?= $this->get_height() ?>px" multiple >
                            <?= $data_source[2] ?>
                        </select>
                    </td>
                </tr>
                <tr>
                    <td colspan=3>
                        <input type="hidden" id="<?= parent::get_id() ?>"  name="<?= parent::get_id() ?>" value="<?= parent::get_value() ?>" class="<?= $this->get_valid() ?> multilist" title="<?= parent::get_title() ?>">
                    </td>
                </tr>
            </table>
        </div>

        <?php
        $html = ob_get_contents();
        ob_clean();

        //listed to listed selected
        $str .= '         <script>
                                    jQuery(\'#' . parent::get_id() . 'add\').bind(\'click\',function(){
                                    moveDualList(document.getElementById(\'' . parent::get_id() . 'multilist_listed\') ,document.getElementById(\'' . parent::get_id() . 'multilist_selected\'),false);
                                    })
                                    jQuery(\'#' . parent::get_id() . 'addall\').bind(\'click\',function(){
                                    moveDualList(document.getElementById(\'' . parent::get_id() . 'multilist_listed\') ,document.getElementById(\'' . parent::get_id() . 'multilist_selected\'),true);
                                    })
                                    jQuery(\'.' . parent::get_id() . 'addbutton\').bind(\'click\',function(){
                                    SelectAllOptions(document.getElementById(\'' . parent::get_id() . 'multilist_selected\') ,document.getElementById(\'' . parent::get_id() . '\'));
                                    })
                                </script>';
        if ($this->get_filter()) {
            $str .= '<script>
                                    var ' . parent::get_id() . 'filter = new filterlist(document.getElementById(\'' . parent::get_id() . 'multilist_listed\'));
                                       jQuery(\'#' . parent::get_id() . 'filterinput\').bind(\'keyup\',function(){
                                                    ' . parent::get_id() . 'filter.set(this.value)
                                       })
                                       jQuery(\'.' . parent::get_id() . 'addbutton\').bind(\'click\',function(){
                                                  ' . parent::get_id() . 'filter.reloadadd(document.getElementById(\'' . parent::get_id() . '\').value,document.getElementById(\'' . parent::get_id() . 'filterinput\').value);
                                       })
                               </script>';
        }
        //selected to listed
        $str .= '
                               <script>
                                     jQuery(\'#' . parent::get_id() . 'delete\').bind(\'click\',function(){';
        if ($this->get_filter())
            $str .= '                      var temparray=' . parent::get_id() . 'filter.setlist(document.getElementById(\'' . parent::get_id() . 'multilist_selected\'));';
        $str .= '                      moveDualList(document.getElementById(\'' . parent::get_id() . 'multilist_selected\') ,document.getElementById(\'' . parent::get_id() . 'multilist_listed\'),false);';
        if ($this->get_filter())
            $str .= parent::get_id() . 'filter.reloaddelete(temparray,document.getElementById(\'' . parent::get_id() . 'filterinput\').value);';
        $str .= '                     })
                                    jQuery(\'#' . parent::get_id() . 'deleteall\').bind(\'click\',function(){';
        if ($this->get_filter())
            $str .= '                     var temparray=' . parent::get_id() . 'filter.setlist(document.getElementById(\'' . parent::get_id() . 'multilist_selected\'),\'true\');';
        $str .= '                     moveDualList(document.getElementById(\'' . parent::get_id() . 'multilist_selected\') ,document.getElementById(\'' . parent::get_id() . 'multilist_listed\'),true);';
        if ($this->get_filter())
            $str .= parent::get_id() . 'filter.reloaddelete(temparray,document.getElementById(\'' . parent::get_id() . 'filterinput\').value);';
        $str .= '                     })
                                    jQuery(\'.' . parent::get_id() . 'deletebutton\').bind(\'click\',function(){
                                       SelectAllOptions(document.getElementById(\'' . parent::get_id() . 'multilist_selected\') ,document.getElementById(\'' . parent::get_id() . '\'));
                                    })
                           </script>';

        if (parent::get_disable()) {
            anti_inspection::push_disabled_elments(parent::get_name(), parent::get_value(),"multilist");
        }
        return $html . $str;
    }

}
?>