<?php

/**
 * form parser_table class . use this class to create tables
 * @author Ihab A. Ali
 * @copyright copyright (c) copyright 2009
 * @license GNU Public Licence (GPL)
 */
class table {

    /**
     * to initiate how many elements per row we have
     * @access private
     * @var int $elements_per_row
     */
    private $elements_per_row = 1;

    /**
     * used internally to store the rows table
     * @access private
     * @var string $table_html
     */
    private $table_html = null;

    /**
     * used internally to store the columns inside row
     * @access private
     * @var string $row_html
     */
    private $row_html = null;

    /**
     * used internally to keep tracking of columns number insdie a row
     * @access private
     * @var int columns numbers
     */
    private $column_no = null;

    /**
     * used to set up row_span i used an array
     * in case the row_span is bigger than 2
     * in order to track to colspan for the next rows
     * @access private
     * @var array  $row_span_cols
     */
    private $row_span_cols = array();

    /**
     * to set up the label align
     * @var string  $label_align
     */
    private $label_align = DIR_RALIGN;

    /**
     * to set up the html elements align
     * @access private
     * @var string $element_align
     */
    private $element_align = DIR_ALIGN;

    /**
     * to set up the valign of a row
     * @access private
     * @var string  $valign
     */
    private $valign = 'middle';

    /**
     * @access private
     * @var in $row_span
     */
    private $row_span = 1;

    /**
     * @access private
     * @var int $col_span
     */
    private $col_span = 1;

    /**
     * to create any new attribute
     * element attr
     * @access private
     * @var array attr
     */
    private $attr = array();

    /**
     * to create any new styling
     * element css
     * @access private
     * @var array css
     */
    private $css = array();

    /**
     * used to set the title of the feildset
     * @access private
     * @var string $title
     */
    private $title = null;

    /**
     * used to set the title of the feildset
     * @access private
     * @var int $width
     */
    private $spacer_width = 10;

    public function __construct() {
        if (dir)
            $this->set_dir(dir);
        $this->set_cell_padding(15);
        $this->set_cell_spacing(15);
        $this->set_border(0);
        $this->set_width('100%');
    }

    /**
     * set table width
     * @access public
     * @param string $width
     */
    public function set_width($width) {
        if ($width == '100%' || $width == '99%')
            $width = '98%';
        $this->attr['width'] = $width;
        return $this;
    }

    /**
     * get table width
     * @access public
     * @return $this->width
     */
    public function get_width() {
        return $this->attr['width'];
    }

    /**
     * set table cellpadding
     * @access public
     * @param int $cell_padding
     */
    public function set_cell_padding($cell_padding) {
        $this->attr['cellpadding'] = $cell_padding;
        return $this;
    }

    /**
     * get table cellpadding
     * @access public
     * @return int $cell_padding
     */
    public function get_cell_padding() {
        return $this->attr['cellpadding'];
    }

    /**
     * set tables cellspacing
     * @access public
     * @param int $cell_spacing
     */
    public function set_cell_spacing($cell_spacing) {
        $this->attr['cellspacing'] = $cell_spacing;
        return $this;
    }

    /**
     * get tables cellspacing
     * @access public
     * @return int $cell_spacing
     */
    public function get_cell_spacing() {
        return $this->attr['cellspacing'];
    }

    /**
     * set tables border
     * @access public
     * @param int $cell_spacing
     */
    public function set_border($border) {
        $this->attr['border'] = $border;
        return $this;
    }

    /**
     * get tables border
     * @access public
     * @return int $border
     */
    public function get_border() {
        return $this->attr['border'];
    }

    /**
     * set the direction of the page rtl for arabic
     * default is ltr -> english
     * saves object inside a $_SESSION['search_object']
     * @access public
     * @param $dir;
     * @return none;
     */
    public function set_dir($dir = null) {
        if ($dir == 'rtl') {
            $this->attr['dir'] = $dir;
            $this->set_label_align('left');
            $this->set_element_align('right');
        } else {
            $this->attr['dir'] = 'ltr';
            $this->set_label_align('right');
            $this->set_element_align('left');
        }
        return $this;
    }

    /**
     * returns the direction of the page
     * @access public
     * @param none;
     * @return string $dir;
     */
    public function get_dir() {
        return $this->attr['dir'];
    }

    /**
     * set the align of lables
     * default = right for english format
     * use left for arabic format
     * @access public
     * @param string $label_align;
     * @return none;
     */
    public function set_label_align($label_align = "right") {
        $this->label_align = $label_align;
        return $this;
    }

    /**
     * get the align of lables
     * default = right for english format
     * use left for arabic format
     * @access public
     * @param none;
     * @return string $label_align;
     */
    public function get_label_align() {
        return $this->label_align;
    }

    /**
     * set the align of html form elements (input,select,radio ...)
     * default = left for english format
     * use right for arabic format
     * @access public
     * @param string $element_align;
     * @return none;
     */
    public function set_element_align($element_align = "left") {
        $this->element_align = $element_align;
        return $this;
    }

    /**
     * returns the align of html form elements (input,select,radio ...)
     * default = left for english format
     * use right for arabic format
     * @access public
     * @param none;
     * @return string $element_align;
     */
    public function get_element_align() {
        return $this->element_align;
    }

    /**
     * set the vertical align of html form elements (label,input,select,radio ...)
     * default = top
     * @access public
     * @param string $valign;
     * @return none;
     */
    public function set_valign($valign = "middle") {
        $this->valign = $valign;
        return $this;
    }

    /**
     * returns the vertical align of html form elements (label,input,select,radio ...)
     * default = top
     * @access public
     * @param none;
     * @return string $valign;
     */
    public function get_valign() {
        return $this->valign;
    }

    /**
     * defines the col and row span of the table
     * columns in this case is created from (2 elements label and elements)
     * @access public
     * @param int $col_span number of spaned columns only 3 to span 2 columns of elements
     * @param int $row_span number of spaned rows
     * @return none;
     */
    public function span($col_span = null, $row_span = null) {
        if ($col_span > 1)
            $this->set_col_span($col_span);
        else
            $this->set_col_span(1);

        if ($row_span > 1)
            $this->set_row_span($row_span);
        else
            $this->set_row_span(1);
        return $this;
    }

    /**
     * defines if the row span of the table
     * @access public
     * @param int $row_span number of spaned rows
     * @return none;
     */
    public function set_row_span($row_span = 1) {
        $this->row_span = $row_span;
        return $this;
    }

    /**
     * returns the row span of the table
     * @access public
     * @param none
     * @return int $row_span number of spaned rows
     */
    public function get_row_span() {
        return $this->row_span;
        return $this;
    }

    /**
     * defines the col span of the table
     * @access public
     * @param int $col_span number of spaned columns only 3 to span 2 columns of elements
     * @return none;
     */
    public function set_col_span($col_span = 1) {
        $this->col_span = $col_span;
        return $this;
    }

    /**
     * defines the row span of the table
     * @access public
     * @param none
     * @return int $col_span number of spaned columns only 3 to span 2 columns of elements
     */
    public function get_col_span() {
        return $this->col_span;
    }

    /**
     * defines the title of the feildset
     * @access public
     * @param string $title
     * @return none;
     */
    public function set_title($title = null) {
        $this->title = $title;
        return $this;
    }

    /**
     * @access public
     * @param none
     * @return string $title
     */
    public function get_title() {
        return $this->title;
    }

    /**
     * defines the number of elements inside a table row
     * by default its set to 2
     * element in this case stands for an object composed of a label , html element or elements (like select,input,radio,fileupload ...)
     * the label will be inside a column and the html element inside another column
     * for a developer the object is considered one column .
     * this logic is used in colspan and setspan by calling set_col_span() or span functions
     * colspan 3 means span 2 object or elements .
     * @access public
     * @param string $page page name like page.php or sub_menu.php
     */
    public function set_elements_per_row($number = 1) {
        $this->elements_per_row = $number;
        return $this;
    }

    /**
     * returns the number of objects or elements per each table row
     * @access private
     * @return int $this->elements_per_row
     */
    private function get_elements_per_row() {
        return $this->elements_per_row;
    }

    /**
     * function get_offered_column_no()
     * returns the real number of offered columns per a table row
     * @access public
     * @return int ($this->get_elements_per_row() * 3)
     */
    private function get_offered_column_no() {
        return ($this->get_elements_per_row() * 3);
    }

    /**
     * function add_column
     * this function adds a column to the table and specifies it as none available
     * a column in this case is an object composed of a several columns depends on the case
     * in normal cases the column is composed of a lable column and an html element column
     * @access private
     * @param int $number
     */
    private function add_column($number) {
        $this->column_no +=$number;
        return $this;
    }

    /**
     * function get_used_col_no() returns the number of used table columns
     * @access public
     * @return int $this->column_no
     */
    private function get_used_col_no() {
        return $this->column_no;
    }

    /**
     * function add_row_span
     * adds the number of row_spans to an array() . each colmun row span is a calculation of the rowspan + colspan
     * @access private
     * @param int $number
     */
    private function add_row_span($number = 0) {
        for ($i = 0; $i < ($this->get_row_span() - 1); $i++)///in case we have row_span set the number of span in an array for the coming rows
            $this->row_span_cols[$i] += $number;
        return $this;
    }

    /**
     * function get_unused_col_no()
     * @access public
     * @return int $number number of unused_col_no
     */
    public function get_unused_col_no() {
        $col_span = 3 * $this->get_col_span(); ///get the real columns number inside the table row
        $available_cols = $this->get_offered_column_no() - $this->get_used_col_no(); ///get the avaible columns number (total number or columns - used colmuns)

        if ($col_span > $available_cols)
            $col_span = $available_cols; /// if the col_span number is bigger than the available columns no we uses the available columns no

        return $col_span;
    }

    /**
     * this function unsets the class attribute of the table_html
     * function unset_table_html
     * @access private
     */
    private function unset_table_html() {
        unset($this->table_html);
        return $this;
    }

    /**
     * this function unsets the class attribute of the row_html
     * function unset_row_html
     * @access private
     */
    private function unset_row_html() {
        unset($this->row_html);
        return $this;
    }

    /**
     * this function unsets the class attribute of the column_no
     * function unset_column_no
     * @access private
     */
    private function unset_column_no() {
        $this->column_no = 0;
        return $this;
    }

    /**
     * this function unsets the class attribute of the row_span_cols
     * function unset_row_span_cols
     * @access private
     */
    private function unset_row_span_cols() {
        unset($this->row_span_cols);
        return $this;
    }

    /**
     * @access private
     * @param string $table_html
     */
    private function add_to_table_html($table_html = null) {
        $this->table_html .= $table_html;
        return $this;
    }

    /**
     * @access private
     * @return string $this->table_html
     */
    private function get_table_html() {
        return $this->table_html;
    }

    /**
     * @access private
     * @param string $row_html
     */
    private function add_to_row_html($row_html = null) {
        $this->row_html .= $row_html;
        return $this;
    }

    /**
     * @access private
     * @return string $this->row_html
     */
    private function get_row_html() {
        return $this->row_html;
    }

    /**
     * @access public
     * @@param int $spacer_width
     */
    public function set_spacer_width($spacer_width = 10) {
        $this->spacer_width = $spacer_width;
        return $this;
    }

    /**
     * @access public
     * @return string $spacer_width;
     */
    public function get_spacer_width() {
        return $this->spacer_width;
    }

    /**
     * function new_element()
     * this function set the html elements inside the table real html columns and saves it inside a class attribute
     * those columns will be later on set inside table rows and loaded when table is laoded
     * in case the form is not set inside table the function returns the element as a string variable
     * @access public
     * @param string $label the label of the object
     * @param string $element the html element (ex:input) of the object
     * @param string $element_type this variable is used inside this function only to set the object properly
     * @return string $variable
     */
    public function new_element($label = 'unlabeled', $element = null, $columns = 3) {
        $col_span = $this->get_unused_col_no();
        switch ($columns) {
            case 3:
                if (!is_object($label))
                    $label = new label($label);
                $cols = "\t\t" . '<td nowrap rowspan="' . $this->get_row_span() . '" align="' . $this->get_label_align() . '">' . $label->load() . '</td>' . "\n\t\t\t\t\t\t";
                $cols .='<td nowrap colspan="' . ($col_span - 2) . '" rowspan="' . $this->get_row_span() . '" align="' . $this->get_element_align() . '">' . $element . '</td>' . "\n\t\t\t\t\t\t";
                $cols .='<td nowrap rowspan="' . $this->get_row_span() . '"><div style="width:' . $this->get_spacer_width() . 'px;height:10px"></div></td>' . "\n\t\t\t\t\t\t";
                $this->add_column($col_span);
                break;
            case 2:
                $cols = '<td nowrap colspan="' . ($col_span - 1) . '" rowspan="' . $this->get_row_span() . '" align="center">' . $element . '</td>
                          <td rowspan="' . $this->get_row_span() . '"><div style="width:' . $this->get_spacer_width() . 'px;height:10px"></div></td>';
                $this->add_column($col_span);
                break;
            case 1:
                $cols = '<td nowrap colspan="' . $col_span . '" rowspan="' . $this->get_row_span() . '">' . $element . '</td>';
                $this->add_column($col_span);
                break;
        }
        $this->add_row_span($col_span);
        $this->span(1, 1);
        $this->add_to_row_html($cols);
        $this->build_row();
        return $this;
    }

    /**
     * this function loads the row and set it inside a class attribute
     * the rows will be loaded later on when the table loads
     * this function computes and regulate the col and row span automatically
     * @access private
     * @return string
     */
    private function build_row() {

        if ($this->get_used_col_no() >= $this->get_offered_column_no()) { //in case the number of colmuns in the row is = to the user defined col number                        //end the current row than open a new row tag
            $html = '
                                    <tr valign="' . $this->get_valign() . '">
                                    ' . $this->get_row_html() . '
                                                <td width=100%>&nbsp;</td>
                                    </tr>
                    ';
            $this->add_to_table_html($html);

            $this->unset_row_html(); // unset the row inner html variable on each new row start
            $this->unset_column_no();

            while ($this->row_span_cols[0] >= $this->get_offered_column_no()) { //in case of row_span where the span is eqaul to the total user defined col number                  //the loop draws an new empty row (the row_span effect)
                $html = '
                                    <tr>
                                                <td width=100%>&nbsp;</td>
                                    </tr>
                ';
                $this->add_to_table_html($html);
                array_shift($this->row_span_cols);
            }
            if ($this->row_span_cols[0]) { // set the row_span number in the new col_span variable
                $this->add_column($this->row_span_cols[0]);
                array_shift($this->row_span_cols);
            }
            return true;
        }
        return false;
    }

    /**
     * function empty_row creates an emtpy row
     * @param string $ln_html
     */
    public function empty_row($ln_html = null) {
        $this->ln();
        if (!$ln_html)
            $ln_html = '&nbsp;';
        $html = '<tr valign="' . $this->get_valign() . '">
            <td colspan="' . $this->get_offered_column_no() . '">' . $ln_html . '</td><td width="100%">&nbsp;</td>
        </tr>';
        $this->add_to_table_html($html);
        return $this;
    }

    /**
     * function empty_row creates an emtpy row
     * @param string $ln_html
     */
    public function hidden_row($ln_html = null) {
        if (!$ln_html)
            $ln_html = '&nbsp;';
        $html = '<tr valign="' . $this->get_valign() . '" style="display:none;">
            <td colspan="' . $this->get_offered_column_no() . '">' . $ln_html . '</td><td width="100%">&nbsp;</td>
        </tr>';
        $this->add_to_table_html($html);
        return $this;
    }

    /**
     * new line
     * @access public
     */
    public function ln() {
        
        if ($this->get_used_col_no() > 0) {
            $this->add_to_table_html(
                    '<tr valign="' . $this->get_valign() . '">
                    ' . $this->get_row_html() . '
                    <td width=100% colspan = "' . ($this->get_unused_col_no() + 1) . '">&nbsp;</td>
                </tr>');
            $this->unset_column_no();
            $this->row_html = "";
        }
        return $this;
    }

    /**
     * add attributes
     * @access public
     * @param string $key
     * @param string $value
     */
    public function add_attr($key, $value) {
        $this->attr[$key] = $value;
        return $this;
    }

    /**
     * get attributes
     * @access public
     * @return string $attributes
     */
    private function get_attr() {
        foreach ($this->attr as $key => $value) {
            $attributes .= $key . '="' . $value . '" ';
        }
        return $attributes;
    }

    /**
     * add css styles
     * @access public
     * @param string $key
     * @param string $value
     */
    public function add_css($key, $value) {
        $this->css[$key] = $value;
        return $this;
    }

    /**
     * get styles
     * @access public
     * @return string $css
     */
    private function get_css() {
        $styles = 'style="';
        foreach ($this->css as $key => $value) {
            $styles .="$key:$value;";
        }
        $styles .='"';
        return $styles;
    }

    /**
     * add js scripts
     * @access public
     * @param string $key
     * @param string $value
     */
    public function add_event($key, $value) {
        $this->attr[$key] = $value;
        return $this;
    }

    /**
     * function load table
     * @param string $title in case we need to set a title
     * @return string $html a php string variable contains the html code of the table
     */
    public function load() {

        if ($this->get_table_html()) {
            $this->ln();

            if ($this->get_title() AND $this->get_title() != "table_title")
                $html = '<fieldset><legend>' . $this->get_title() . '</legend>';

            $html .='<table ' . $this->get_attr() . ' ' . $this->get_css() . '>';
            $html .= $this->get_table_html();
            $html .='</table>';
            if ($this->get_title())
                $html.='</fieldset>';
            $this->unset_row_html();
            $this->unset_column_no();
            $this->unset_row_span_cols();
            $this->unset_table_html();
        }
        return $html;
    }

}

?>