<?php

/**
 * html parser class . use this class to create the outer form and tabs ....
 * @author Ihab A. Ali
 * @copyright copyright (c) copyright 2009
 * @license GNU Public Licence (GPL)
 * @example  $theme = new theme()
 */
class theme {

    /**
     * set the width of the xp style frames
     * @access private
     * @var int $width
     */
    private $width = '100%';

    /**
     * set the height of the xp style frames
     * @access private
     * @var int $height
     */
    private $height = '100%';

    /**
     * set the title of the xp style frames
     * @access private
     * @var string $title
     */
    private $title = 'untitled';

    /**
     * in case of none tabs forms we use this variable to initialize the InnerHTML
     * @access private
     * @var string $html
     */
    private $html = null;

    /**
     * @access private
     * @var bool $show_message
     */
    private $show_message = false;

    /**
     * @access private
     * @var string $dir
     */
    private $dir = 'ltr';

    /**
     * @access  private
     * @var string $text_ralign
     */
    private $text_ralign = 'right';

    /**
     * usetd to set the xp style texts align default is center , for arabic set to right ,english set to left
     * @access private
     * @var string $text_align
     */
    private $text_align = 'left';

    /**
     * used to set the xp style frames align default is center , for arabic set to right ,english set to left
     * @access private
     * @var string $align
     */
    private $align = 'center';

    /**
     *
     * @var string scripts
     */
    private $scripts = array();

    /**
     *
     * @var string css
     */
    private $css = array();

    /**
     * class __construct
     * @access public
     */
    public function __construct() {
        if (dir) {
            $this->set_dir(dir);
        }
    }

    /**
     * set dir
     * @access public
     * @param string $dir
     */
    public function set_dir($dir = 'ltr') {
        $this->dir = $dir;
        switch ($dir) {
            case 'rtl':
                $this->text_align = 'right';
                $this->text_ralign = 'left';
                break;
            default:
                $this->text_align = 'left';
                $this->text_ralign = 'right';
        }
        return $this;
    }

    /**
     * @access public
     * @param bool $bool
     */
    public function disable_context_menu($bool = true) {
        if ($bool) {
            $this->add_script('document.oncontext_menu=new Function("return false");');
        }
        return $this;
    }

    /**
     * @access public
     * @param string $title
     */
    public function set_title($title = 'untitled') {
        $this->title = $title;
        return $this;
    }

    /**
     * @access public
     * @return string $title
     */
    public function get_title() {
        return $this->title;
    }

    /**
     * @access public
     * @param int $height
     */
    public function set_height($height = '100%') {
        $this->height = $height;
        return $this;
    }

    /**
     * @access public
     * @return int $height
     */
    public function get_height() {
        return $this->height;
    }

    /**
     * @access public
     * @param int $width
     */
    public function set_width($width = '100%') {
        $this->width = $width;
        return $this;
    }

    /**
     * @access public
     * @return int $width
     */
    public function get_width() {
        return $this->width;
    }

    /**
     * @access public
     * @param string $align
     */
    public function set_align($align = 'center') {
        $this->align = $align;
        return $this;
    }

    /**
     * @access public
     * @return string $align
     */
    public function get_align() {
        return $this->align;
    }

    /**
     * @access public
     * @return string dir
     */
    public function get_dir() {
        return $this->dir;
    }

    /**
     * @access public
     * @return string text_ralign
     */
    public function get_text_ralign() {
        return $this->text_ralign;
    }

    /**
     * @access public
     * @return string $text_align
     */
    public function get_text_align() {
        return $this->text_align;
    }

    /**
     *
     * @param string $html
     */
    public function set_html($html = null) {
        $this->html = $html;
        return $this;
    }

    /**
     *
     * @return string $html
     */
    public function get_html() {
        return $this->html;
    }

    /**
     * @param string $add_script
     */
    public function add_script($script = null) {
        array_push($this->scripts, $script);
        return $this;
    }

    /**
     *
     * @return string $script
     */
    public function get_script() {
        $scripts = null;
        foreach ($this->scripts as $value) {
            $scripts .= "
                        " . $value;
        }
        return $scripts;
    }

    /**
     * @param string $add_css
     */
    public function add_css($css = null) {
        array_push($this->css, $css);
        return $this;
    }

    /**
     *
     * @return string $css
     */
    public function get_css() {
        $csss = null;
        foreach ($this->css as $value) {
            $csss .= "
                        " . $value;
        }
        return $csss;
    }

    /**
     *
     * @param string $value
     * @return bool
     */
    public function is_percentage($value) {
        return preg_match("/%/", $value);
    }

    /**
     * @access private
     * @param bool $bool
     */
    public function show_message($bool) {
        $this->show_message = $bool;
        return $this;
    }

    /**
     * return bool $show_message
     */
    private function is_message_shown() {
        if ($this->show_message) {
            return '';
        }
        return 'none';
    }

    /**
     * 
     */
    public function load() {
        $ccache="";
        if(force_overide_cache == "1"){
            $ccache="?time=".time();
        }
        ?>
        <!DOCTYPE html>
        <html>
            <head>
                <title><?= $this->get_title() ?></title>
                <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
                <link rel="stylesheet" href="<?= theme_common_url_path ?>css/roboto.css?family=Roboto:300,500,700&display=swap" />
                <link rel="stylesheet" href="<?= theme_common_url_path ?>css/mdi.min.css<?=$ccache?>"/>
                <link rel="stylesheet" href="<?= theme_common_url_path ?>css/message.css<?=$ccache?>"/>
                <link rel="stylesheet" href="<?= theme_common_url_path ?>css/mdil.min.css<?=$ccache?>" />
                <link rel="stylesheet" href="<?= plugins_url_path ?>/combo/dhtmlxcombo.css<?=$ccache?>" type="text/css" />                
                <link rel="stylesheet" href="<?= theme_common_url_path ?>css/template.css<?=$ccache?>" type="text/css" />                
                <link rel="stylesheet" href="<?= theme_common_url_path ?>css/controls.css<?=$ccache?>" type="text/css" />                
                <link rel="stylesheet" href="<?= theme_common_url_path ?>css/grid.css<?=$ccache?>" type="text/css" />
                <link rel="stylesheet" href="<?= theme_common_url_path ?>css/ajax_tree.css<?=$ccache?>" type="text/css" />
                <link rel="stylesheet" href="<?= theme_common_url_path ?>css/tab.css<?=$ccache?>" type="text/css" />
                <link rel="stylesheet" href="<?= theme_common_url_path ?>css/context_menu.css<?=$ccache?>" />
                <link rel="stylesheet" href="<?= theme_common_url_path ?>css/datepicker.css<?=$ccache?>" type="text/css" />
                <link rel="stylesheet" href="<?= theme_common_url_path ?>css/jquery.tipsy.css<?=$ccache?>" />
                <link rel="stylesheet" href="<?= theme_common_url_path ?>css/notiflix-2.1.2.min.css<?=$ccache?>" />
                <?php include theme_common_physical_path.'definesjs.php';?>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/jquery.min.js<?=$ccache?>"></script>                                
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/notiflix-2.1.2.min.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/jquery.tipsy.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/validate.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/multilist.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/filterlist.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/jquery.cookie.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/datepicker.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/message.js<?=$ccache?>"></script>                
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/jquery_ext.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/tab.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/template.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/grid.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/new_window.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/jquery.metadata.min.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/jquery.json-2.2.min.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/jquery.rightClick.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/money_format.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/context_menu.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/ajax_tree.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/jquery.calculation.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/jquery.hotkeys.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= plugins_url_path ?>/combo/dhtmlxcombo.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/keybord.shortcuts.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/fullscreen.api.js<?=$ccache?>"></script>
                <script type="text/javascript" src="<?= theme_common_url_path ?>js/octacheckbox.js<?=$ccache?>"></script>
                <style>
        <?= $this->get_css() ?>
                </style>                
            </head>
            <body class="parent_iframe_body direction_<?= $this->get_dir() ?>">
                <header>
                    <div class="form_title_div form_title_<?= $this->get_dir() ?>"><?= $this->get_title() ?></div>
                </header>
                <div class="work_space">
                    <?= $this->get_html() ?>
                </div>
                <div class="work_space work_space_shadow"></div>
            </body>
        </html>
        <?php
    }

}
?>
