<?php
// require_once '../../../ap/framework/connecter.php';

class Path
{
	static function isEmpty($str)
	{
		return $str == '';
	}

	static function normalizeArray($parts, $allowAboveRoot)
	{
		$res = [];
		foreach ($parts as $p) {
			// ignore empty parts
			if (Path::isEmpty($p) || $p === '.')
				continue;

			if ($p === '..') {
				$resCount = count($res);

				if ($resCount && $res[$resCount - 1] !== '..') {
					array_pop($res);
				} else if ($allowAboveRoot) {
					$res[] = '..';
				}

				continue;
			}
			$res[] = $p;
		}

		return $res;
	}
	private static function posixIsAbsolute($path)
	{
		return $path && $path[0] === '/';
	}
	private static function makeAbsolute($path) {
		if (self::posixIsAbsolute($path)) return $path;

		return "/$path";
	}
	private static function posixNormalize($path)
	{
		$isAbsolute = self::posixIsAbsolute($path);
		$path = implode('/', self::normalizeArray(explode('/', $path), !$isAbsolute));

		if (Path::isEmpty($path) && !$isAbsolute) {
			$path = '.';
		}

		return $isAbsolute ? self::makeAbsolute($path) : $path;
	}
	static function posixJoin(...$paths)
	{

		$currentPath = '';
		foreach ($paths as $segment) {
			if (Path::isEmpty($segment)) {
				continue;
			}

			if (Path::isEmpty($currentPath)) {
				$currentPath .= $segment;
			} else {
				$currentPath .= "/$segment";
			}
		}


		return self::posixNormalize($currentPath);
	}
	static function posixEquals($a, $b)
	{
		$a = self::posixNormalize($a);
		$b = self::posixNormalize($b);

		return $a == $b;
	}

	static function normalize($path)
	{
		return self::posixNormalize($path);
	}
	static function join(...$paths)
	{
		return self::posixJoin(...$paths);
	}
	static function joinURL($url, ...$paths)
	{
		$parsedUrl = self::parse($url);
		$parsedUrl['path'] = self::posixJoin($parsedUrl['path'], ...$paths);

		return self::buildURL($parsedUrl);
	}
	static function equals($a, $b)
	{
		return self::posixEquals($a, $b);
	}


	static function info($path, int $flags = PATHINFO_ALL)
	{
		return pathinfo(self::getPathName($path), $flags);
	}
	static function parse($path, int $component = -1)
	{
		return parse_url($path, $component);
	}
	static function buildURL($parsedUrl)
	{ 
		@list(
			"scheme" => $scheme,
			"user" => $user,
			"pass" => $pass,
			"host" => $host,
			"port" => $port,
			"path" => $path,
			"query" => $query,
			"fragment" => $fragment,
		) = $parsedUrl;
		$url = $scheme ? $scheme . '://' : '';

		if (isset($user)) {
			$url .= $user;
			$url .= isset($pass) ? ':' . $pass : '';
			$url .= '@';
		}

		$url .= isset($host) ? $host : '';

		$url .= isset($port) ? ':' . $port : '';

		if (isset($path)) {
			$url .= ($path[0] !== '/') ? '/' . $path : $path;
		}

		$url .= isset($query) ? '?' . $query : '';
		$url .= isset($fragment) ? '#' . $fragment : '';

		return $url;
	}
	static function directory($path)
	{
		return dirname($path);
	}

	static function getPathName($path)
	{

		return self::parse($path, PHP_URL_PATH);
	}
	static function getNormalizedPathName($path)
	{

		return self::normalize($path);
	}
	static function query($path)
	{
		return self::parse($path, PHP_URL_QUERY);
	}

	static function extension($path)
	{
		return self::info($path, PATHINFO_EXTENSION);
	}
	static function filename($path)
	{
		return self::info($path, PATHINFO_FILENAME);
	}
	static function basename($path)
	{
		return self::info($path, PATHINFO_BASENAME);
	}

	static function absoluteToUpload($path)
	{
		return Path::join(upload_physical_path, $path);
	}
}